##
#
# Migration cost evaluation script.
#
# Send one service request during the setup phase (first 60s of the
# simulation), then migrate the service, then wait 10s, and send another
# service request during the next 60s. Note that 60s corresponds to the
# SLP_MAX_CACHE_ENTRY_AGE in slp_cache.c.
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: migration_cost_evaluation.tcl 3380 2010-03-02 17:08:01Z gwitten $
#
##



source common/config.tcl

# Number of mobile nodes.
set val(nn)	%%nodes%%

# Size of topography (m) is initialized while setting up nodes.
#set val(x)	1000
set val(x)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]
#set val(y)	1000
set val(y)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]

# Simulation setup duration (s).
#set simulation_setup_duration	0.0	;# 0 minutes
set simulation_setup_duration	60.0	;# 1 minute
#set simulation_setup_duration	300.0	;# 5 minutes
#set simulation_setup_duration	900.0	;# 15 minutes
#set simulation_setup_duration	3600.0	;# 1 hour

# Simulation duration (s).
#set simulation_duration	0.0	;# 0 minute
#set simulation_duration	60.0	;# 1 minute
set simulation_duration		70.0	;# 1:10 minute
#set simulation_duration	300.0	;# 5 minutes
#set simulation_duration	900.0	;# 15 minutes
#set simulation_duration	3600.0	;# 1 hour



source common/header.tcl

# Create, initialize and connect nodes, agents and applications.
for {set i 0} {$i < $val(nn)} {incr i} {
	set node_($i) [$ns_ node]
#	$node_($i) set X_ $i
	$node_($i) set X_ [$rng_ uniform 0 $val(x)]
#	$node_($i) set X_ [expr ($i % int(sqrt($val(nn)))) * $grid_spacing_distance]
#	$node_($i) set Y_ 0
	$node_($i) set Y_ [$rng_ uniform 0 $val(y)]
#	$node_($i) set Y_ [expr ($i / int(sqrt($val(nn)))) * $grid_spacing_distance]
	$node_($i) set Z_ 0
	$node_($i) random-motion 0		;# Disable random motion
	$god_ new_node $node_($i)
	$ns_ initial_node_pos $node_($i) 10
	puts [format "Placing node #%d at (%.2f, %.2f, %.2f)..." $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_]]

	set agent_($i) [new Agent/MessagePassing]
	set application_($i) [new Application/C]
	$node_($i) attach $agent_($i)
	$application_($i) attach-agent $agent_($i)
	$application_($i) seed heuristic 0	;# Enable heuristic random seed in application

	set start_time($i) [$rng_ uniform $start_phase_begin $start_phase_end]
#	set stop_time($i) [$rng_ uniform $stop_phase_begin $stop_phase_end]
	set stop_time($i) $stop_phase_begin
	$ns_ at $start_time($i) "$application_($i) start"
	$ns_ at $stop_time($i) "$application_($i) stop"
	$ns_ at $finalize_phase_begin "$application_($i) finalize"
	$ns_ at $simulation_end "$application_($i) SERVICE_set_placement_algorithm none"
}
#puts [format "Center node is node #%d at (%.2f, %.2f, %.2f)." $grid_center_node [$node_($grid_center_node) set X_] [$node_($grid_center_node) set Y_] [$node_($grid_center_node) set Z_]]



#
# Service setup.
#

# Service requests per node per second.
set request_frequency %%request_frequency%%

# Active node ratio (0.0 -> all nodes quiet; 1.0 -> all nodes active).
set active_node_ratio %%activity_factor%%

# Synchronization ratio (0.0 -> no synchronization traffic; 1.0 -> synchronization of every request).
set sync_ratio %%sync_ratio%%

# Node to host the first service instance.
#set initial_service_host 0
#set initial_service_host $grid_center_node
set initial_service_host [$rng_ integer $val(nn)]

# Target node for the migration of the service instance.
set migration_target_service_host [$rng_ integer $val(nn)]
while {$migration_target_service_host == $initial_service_host} {
	set migration_target_service_host [$rng_ integer $val(nn)]
}

# Start initial service instance of "srvA".
$ns_ at 0 "$application_($initial_service_host) SERVICE_create_service srvA %%service_size%% $sync_ratio"
#$ns_ at 1 "$application_($initial_service_host) SERVICE_is_local_service srvA"

# Start placement algorithm.
$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA none"


#
# Service requests, migration, and more requests.
#

# Have all active nodes in the network request or update data of "srvA".
for {set i 0} {$i < $val(nn)} {incr i} {
	if {[$rng_ uniform 0.0 1.0] <= $active_node_ratio} {
		set t $start_time($i)
		if {[$rng_ uniform 0.0 1.0] <= $sync_ratio} {
			$ns_ at $t "$application_($i) SERVICE_update_data srvA 0x0000 1024"
			#puts [format "Scheduling service update from node #%d at %.4f..." $i $t]
		} else {
			$ns_ at $t "$application_($i) SERVICE_request_data srvA 0x0000 1024"
			#puts [format "Scheduling service request from node #%d at %.4f..." $i $t]
		}
		set t [$rng_ uniform [expr $simulation_begin + 10.0] $simulation_end]
		if {[$rng_ uniform 0.0 1.0] <= $sync_ratio} {
			$ns_ at $t "$application_($i) SERVICE_update_data srvA 0x0000 1024"
			#puts [format "Scheduling service update from node #%d at %.4f..." $i $t]
		} else {
			$ns_ at $t "$application_($i) SERVICE_request_data srvA 0x0000 1024"
			#puts [format "Scheduling service request from node #%d at %.4f..." $i $t]
		}
	}
}

# Migrate service "srvA" from intial node to target node.
$ns_ at $simulation_begin "$application_($initial_service_host) SERVICE_migrate_service srvA $migration_target_service_host"



# Start simulation.
puts "Starting Simulation..."
$ns_ run
