##
#
# Generic setup header for all simulations.
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: header.tcl 3497 2010-05-06 15:24:51Z gwitten $
#
##



# Create God
set god_ [create-god $val(nn)]
$god_ on

# Create simulator
set ns_ [new Simulator]

# Initialize random number generator with new seed (0 -> random seed)
set rng_ [new RNG]
$rng_ seed 0

# Create channel
set chan_ [new $val(chan)]

# Create topography
set topo_ [new Topography]
$topo_ load_flatgrid $val(x) $val(y)

# Setup node configuration
$ns_ node-config \
	-adhocRouting $val(rp) \
	-llType $val(ll) \
	-macType $val(mac) \
	-ifqType $val(ifq) \
	-ifqLen $val(ifqlen) \
	-antType $val(ant) \
	-propType $val(prop) \
	-phyType $val(netif) \
	-topoInstance $topo_ \
	-agentTrace OFF \
	-routerTrace OFF \
	-macTrace OFF \
	-movementTrace OFF \
	-channel $chan_ \
	-energyModel $val(engmodel) \
	-txPower $val(txPower) \
	-rxPower $val(rxPower) \
	-idlePower $val(idlePower) \
	-initialEnergy $val(initeng)

# Enable heuristic random seed in shadowing radio propagation.
if {[$ns_ set propType_] == "Propagation/Shadowing"} then {
	[$ns_ set propInstance_] seed heuristic 0
}



# Initialize exported 802.11 MAC variables
Mac/802_11		set transmittedPackets_				0
Mac/802_11		set transmissionFailures_			0
Mac/802_11		set receivedPackets_				0
Mac/802_11		set droppedPackets_capture_			0
Mac/802_11		set droppedPackets_collision_			0
Mac/802_11		set droppedPackets_transmit_			0
Mac/802_11		set droppedPackets_bit_error_			0
Mac/802_11		set transmittedBytes_				0
Mac/802_11		set receivedBytes_				0

# Initialize exported C code variables
Application/C		set DEBUG_ERRORs_				0
Application/C		set DEBUG_WARNs_				0
Application/C		set averageNodeDegree_				0.0
Application/C		set averageNodeDistance_			0.0
for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
	Application/C	set [format "unicastPacketsPerHopCount_%d_" $hopCount]	0
}
Application/C		set transmittedUnicastPackets_			0
Application/C		set unicastTransmissionFailures_		0
Application/C		set receivedUnicastPackets_			0
Application/C		set transmittedBroadcastPackets_		0
Application/C		set receivedBroadcastPackets_			0
Application/C		set transmittedBytes_				0
Application/C		set receivedBytes_				0
Application/C		set transmittedDatarate_			0
Application/C		set receivedDatarate_				0
Application/C		set networkMappingOverhead_			0
Application/C		set DYMO_transmittedRREQs_			0
Application/C		set DYMO_transmittedRREPs_			0
Application/C		set DYMO_transmittedRERRs_			0
Application/C		set DYMO_transmittedUnicastPackets_		0
Application/C		set DYMO_receivedUnicastPackets_		0
Application/C		set DYMO_transmittedBroadcastPackets_		0
Application/C		set DYMO_receivedBroadcastPackets_		0
Application/C		set RUDP_totalTransmissions_			0
Application/C		set RUDP_successfulTransmissions_		0
Application/C		set ICMP_transmittedNops_			0
Application/C		set ICMP_receivedNops_				0
Application/C		set SLP_totalQueries_				0
Application/C		set SLP_successfulQueries_			0
Application/C		set SERVICE_totalDataRequests_			0
Application/C		set SERVICE_successfulDataRequests_		0
Application/C		set SERVICE_totalSuccessfulDataRequestDelay_	0.0
Application/C		set SERVICE_totalDataUpdates_			0
Application/C		set SERVICE_successfulDataUpdates_		0
Application/C		set SERVICE_totalStreamRequests_		0
Application/C		set SERVICE_successfulStreamRequests_		0
Application/C		set SERVICE_expectedStreamPackets_		0
Application/C		set SERVICE_receivedStreamPackets_		0
Application/C		set SERVICE_totalRequests_			0
Application/C		set SERVICE_successfulRequests_			0
Application/C		set SERVICE_failedRequests_			0
Application/C		set SERVICE_transmittedBytes_			0
Application/C		set SERVICE_receivedBytes_			0
Application/C		set SERVICE_transmittedDatarate_		0
Application/C		set SERVICE_receivedDatarate_			0
for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
	Application/C	set [format "SERVICE_requestPacketsPerHopCount_%d_" $hopCount]	0
}
Application/C		set SERVICE_currentlyHostedServices_		0
Application/C		set SERVICE_averageHostedServices_		0.0
Application/C		set SERVICE_syncOverheadBytes_			0
Application/C		set SERVICE_totalReplications_			0
Application/C		set SERVICE_successfulReplications_		0
Application/C		set SERVICE_totalMigrations_			0
Application/C		set SERVICE_successfulMigrations_		0
Application/C		set PLACEMENT_currentServiceCost_		0.0
Application/C		set PLACEMENT_currentlyOptimalServiceCost_	0.0
Application/C		set PLACEMENT_currentlyOptimalNumberOfHosts_	0
Application/C		set PLACEMENT_currentlyEstimatedAdaptationCost_ 0.0
Application/C		set PLACEMENT_totalAdaptations_			0
Application/C		set PLACEMENT_successfulAdaptations_		0
Application/C		set PLACEMENT_failedAdaptations_		0



# Open trace files
#set tracefd_	[open out.tr w]
set tracefd_	[open /dev/null w]
$ns_ trace-all $tracefd_
#set namtracefd_ [open out.nam w]
set namtracefd_ [open /dev/null w]
$ns_ namtrace-all-wireless $namtracefd_ $val(x) $val(y)

# Set display speed of NAM trace.
$ns_ puts-nam-traceall "v -t 0.0 -e set_rate_ext 200ms 1"

# Remember start time
set start_time_ [clock seconds]

# Begin and end of start, simulation, stop and finalize phases.
# TODO: Clean up and document the distinctions between the phases.
set start_phase_begin		0.0
set start_phase_end		[expr $start_phase_begin + $simulation_setup_duration]
set simulation_begin		$start_phase_end
set simulation_end		[expr $simulation_begin + $simulation_duration]
set stop_phase_begin		[expr $simulation_end + 60.0]	;# FIXME: This actually adds 60s to all simulations. Remove when evaluation is complete.
set stop_phase_end		[expr $stop_phase_begin + 60.0]
set finalize_phase_begin	[expr $stop_phase_end + 60.0]
set finalize_phase_end		[expr $finalize_phase_begin + 300.0]

# Print some info and statistics as the simulation runs.
$ns_ at $start_phase_begin "printStats false true false"
for {set t $start_phase_begin} {$t <= $stop_phase_end} {set t [expr $t + 1]} {

	# Add evenly timed ticks to the output.
	$ns_ at $t "puts [format \"\\\[%.4f\\\]\" $t]"

	# Print statistics for the current point in time.
	$ns_ at $t "printStats false false true"
}

$ns_ at [expr $finalize_phase_end] "stop"



# Calculate the center node for a grid layout.
if {[expr $val(nn) % 2] == 0} {
	set grid_center_node	[expr $val(nn) / 2 - int(sqrt($val(nn))) / 2 - 1]
} else {
	set grid_center_node	[expr $val(nn) / 2]
}



proc stop {} {
	global ns_ tracefd_ namtracefd_ start_time_
	$ns_ halt
	$ns_ nam-end-wireless [$ns_ now]
	$ns_ flush-trace
	close $tracefd_
	close $namtracefd_
	set elapased_time [expr [clock seconds] - $start_time_ - 3600]	;# Formatting miraculously adds one hour.
	puts [format "Simulation complete (%s)." [clock format $elapased_time -format "%T"]]
#	puts ""
#	printProcessStatus
	puts ""
	printStats true true true
	puts "\7"	;# Ring system bell when done.
}

proc printProcessStatus {} {
	set stats [open /proc/self/status r]
	while {[gets $stats line] >= 0} {
		puts $line
	}
	close $stats
}

# Adapted from http://code.activestate.com/recipes/146220/.
proc thousands {num {sep ,}} {
	if {round($num) != $num} {
		set num [format "%.3f" $num]
	}
	while {[regsub {^([-+]?\d+)(\d\d\d)} $num "\\1$sep\\2" num]} {}
	return $num
}

# This variable must not be reset when printing statistics, hence it is
# declared globally.
set SERVICE_availableUntilTime		0.0

proc printStats {printReadableOuput printParseableHeader printParseableResults} {
	global ns_ val maximal_network_diameter simulation_setup_duration simulation_duration application_ node_ SERVICE_availableUntilTime

	set DEBUG_ERRORs				0
	set DEBUG_WARNs					0

	set current_time				[$ns_ now]
	set total_duration				[expr $simulation_setup_duration + $simulation_duration]
	set averageNodeDegree				0.0
	set averageNodeDistance				0.0
	set averageUnicastHopCount			0.0
	set maximalUnicastHopCount			0
	for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
		set unicastPacketsPerHopCount($hopCount)	0
	}
	set transmittedUnicastPackets			0
	set unicastTransmissionFailures			0
	set receivedUnicastPackets			0
	set unicastPDR					0.0
	set transmittedBroadcastPackets			0
	set receivedBroadcastPackets			0
	set broadcastPDR				0.0
	set transmittedBytes				wide(0)
	set receivedBytes				wide(0)
	set currentTransmittedDatarate			0
	set currentReceivedDatarate			0
	set averageTransmittedDatarate			0.0
	set averageReceivedDatarate			0.0
	set overhead_wrt_serviceTraffic_absolute	0
	set overhead_wrt_serviceTraffic_relative	0.0
	set networkMappingOverhead_absolute		0
	set networkMappingOverhead_relative		0.0

	set MAC_transmittedPackets			0
	set MAC_transmissionFailures			0
	set MAC_receivedPackets				0
	set MAC_droppedPackets_capture			0
	set MAC_droppedPackets_collision		0
	set MAC_droppedPackets_transmit			0
	set MAC_droppedPackets_bit_error		0
	set MAC_transmittedBytes			wide(0)
	set MAC_receivedBytes				wide(0)
	set MAC_averageTransmittedDatarate		0.0
	set MAC_averageReceivedDatarate			0.0
	set MAC_overhead_wrt_serviceTraffic_absolute	0
	set MAC_overhead_wrt_serviceTraffic_relative	0.0

	set DYMO_transmittedRREQs			0
	set DYMO_transmittedRREPs			0
	set DYMO_transmittedRERRs			0
	set DYMO_transmittedUnicastPackets		0
	set DYMO_receivedUnicastPackets			0
	set DYMO_unicastPdr				0.0
	set DYMO_transmittedBroadcastPackets		0
	set DYMO_receivedBroadcastPackets		0
	set DYMO_broadcastPdr				0.0

	set RUDP_totalTransmissions			0
	set RUDP_successfulTransmissions		0
	set RUDP_pdr					0.0

	set ICMP_transmittedNops			0
	set ICMP_receivedNops				0
	set ICMP_nopPdr					0.0

	set SLP_totalQueries				0
	set SLP_successfulQueries			0
	set SLP_recall					0.0

	set SERVICE_totalDataRequests			0
	set SERVICE_successfulDataRequests		0
	set SERVICE_dataRequestRecall			0.0
	set SERVICE_totalSuccessfulDataRequestDelay	0.0
	set SERVICE_averageSuccessfulDataRequestDelay	0.0
	set SERVICE_totalDataUpdates			0
	set SERVICE_successfulDataUpdates		0
	set SERVICE_dataUpdateRecall			0.0
	set SERVICE_totalStreamRequests			0
	set SERVICE_successfulStreamRequests		0
	set SERVICE_streamRequestRecall			0.0
	set SERVICE_expectedStreamPackets		0
	set SERVICE_receivedStreamPackets		0
	set SERVICE_streamPacketRecall			0.0
	set SERVICE_totalRequests			0
	set SERVICE_successfulRequests			0
	set SERVICE_requestRecall			0.0
	set SERVICE_transmittedBytes			wide(0)
	set SERVICE_receivedBytes			wide(0)
	set SERVICE_currentTransmittedDatarate		0
	set SERVICE_currentReceivedDatarate		0
	set SERVICE_averageTransmittedDatarate		0.0
	set SERVICE_averageReceivedDatarate		0.0
	set SERVICE_averageRequestHopCount		0.0
	set SERVICE_maximalRequestHopCount		0
	for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
		set SERVICE_requestPacketsPerHopCount($hopCount)	0
	}

	set SERVICE_currentlyHostedServices		0
	set SERVICE_averageHostedServices		0.0
	#set SERVICE_availableUntilTime			0.0	Declared globally (see above).
	set SERVICE_syncOverhead_absolute		wide(0)
	set SERVICE_syncOverhead_relative		0.0
	set SERVICE_totalReplications			0
	set SERVICE_successfulReplications		0
	set SERVICE_replicationSuccessRate		0.0
	set SERVICE_totalMigrations			0
	set SERVICE_successfulMigrations		0
	set SERVICE_migrationSuccessRate		0.0

	set PLACEMENT_currentServiceCost		0.0
	set PLACEMENT_currentlyOptimalServiceCost	0.0
	set PLACEMENT_currentlyOptimalNumberOfHosts	0
	set PLACEMENT_currentlyEstimatedAdaptationCost	0.0
	set PLACEMENT_totalAdaptations			0
	set PLACEMENT_successfulAdaptations		0
	set PLACEMENT_adaptationSuccessRate		0.0

	for {set i 0} {$i < $val(nn)} {incr i} {
		set DEBUG_ERRORs [expr $DEBUG_ERRORs + [$application_($i) set DEBUG_ERRORs_]]
		set DEBUG_WARNs [expr $DEBUG_WARNs + [$application_($i) set DEBUG_WARNs_]]

		set averageNodeDegree [expr $averageNodeDegree + [$application_($i) set averageNodeDegree_]]
		set averageNodeDistance [expr $averageNodeDistance + [$application_($i) set averageNodeDistance_]]
		for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
			set unicastPacketsPerHopCount($hopCount) [expr $unicastPacketsPerHopCount($hopCount) + [$application_($i) set [format "unicastPacketsPerHopCount_%d_" $hopCount]]]
		}
		set transmittedUnicastPackets [expr $transmittedUnicastPackets + [$application_($i) set transmittedUnicastPackets_]]
		set unicastTransmissionFailures [expr $unicastTransmissionFailures + [$application_($i) set unicastTransmissionFailures_]]
		set receivedUnicastPackets [expr $receivedUnicastPackets + [$application_($i) set receivedUnicastPackets_]]
		set transmittedBroadcastPackets [expr $transmittedBroadcastPackets + [$application_($i) set transmittedBroadcastPackets_]]
		set receivedBroadcastPackets [expr $receivedBroadcastPackets + [$application_($i) set receivedBroadcastPackets_]]
		set transmittedBytes [expr $transmittedBytes + [$application_($i) set transmittedBytes_]]
		set receivedBytes [expr $receivedBytes + [$application_($i) set receivedBytes_]]
		set currentTransmittedDatarate [expr $currentTransmittedDatarate + [$application_($i) set transmittedDatarate_]]
		set currentReceivedDatarate [expr $currentReceivedDatarate + [$application_($i) set receivedDatarate_]]
		set networkMappingOverhead_absolute [expr $networkMappingOverhead_absolute + [$application_($i) set networkMappingOverhead_]]

		set MAC_transmittedPackets [expr $MAC_transmittedPackets + [[$node_($i) set mac_(0)] set transmittedPackets_]]
		set MAC_transmissionFailures [expr $MAC_transmissionFailures + [[$node_($i) set mac_(0)] set transmissionFailures_]]
		set MAC_receivedPackets [expr $MAC_receivedPackets + [[$node_($i) set mac_(0)] set receivedPackets_]]
		set MAC_droppedPackets_capture [expr $MAC_droppedPackets_capture + [[$node_($i) set mac_(0)] set droppedPackets_capture_]]
		set MAC_droppedPackets_collision [expr $MAC_droppedPackets_collision + [[$node_($i) set mac_(0)] set droppedPackets_collision_]]
		set MAC_droppedPackets_transmit [expr $MAC_droppedPackets_transmit + [[$node_($i) set mac_(0)] set droppedPackets_transmit_]]
		set MAC_droppedPackets_bit_error [expr $MAC_droppedPackets_bit_error + [[$node_($i) set mac_(0)] set droppedPackets_bit_error_]]
		set MAC_transmittedBytes [expr $MAC_transmittedBytes + [[$node_($i) set mac_(0)] set transmittedBytes_]]
		set MAC_receivedBytes [expr $MAC_receivedBytes + [[$node_($i) set mac_(0)] set receivedBytes_]]

		set DYMO_transmittedRREQs [expr $DYMO_transmittedRREQs + [$application_($i) set DYMO_transmittedRREQs_]]
		set DYMO_transmittedRREPs [expr $DYMO_transmittedRREPs + [$application_($i) set DYMO_transmittedRREPs_]]
		set DYMO_transmittedRERRs [expr $DYMO_transmittedRERRs + [$application_($i) set DYMO_transmittedRERRs_]]
		set DYMO_transmittedUnicastPackets [expr $DYMO_transmittedUnicastPackets + [$application_($i) set DYMO_transmittedUnicastPackets_]]
		set DYMO_receivedUnicastPackets [expr $DYMO_receivedUnicastPackets + [$application_($i) set DYMO_receivedUnicastPackets_]]
		set DYMO_transmittedBroadcastPackets [expr $DYMO_transmittedBroadcastPackets + [$application_($i) set DYMO_transmittedBroadcastPackets_]]
		set DYMO_receivedBroadcastPackets [expr $DYMO_receivedBroadcastPackets + [$application_($i) set DYMO_receivedBroadcastPackets_]]

		set RUDP_totalTransmissions [expr $RUDP_totalTransmissions + [$application_($i) set RUDP_totalTransmissions_]]
		set RUDP_successfulTransmissions [expr $RUDP_successfulTransmissions + [$application_($i) set RUDP_successfulTransmissions_]]

		set ICMP_transmittedNops [expr $ICMP_transmittedNops + [$application_($i) set ICMP_transmittedNops_]]
		set ICMP_receivedNops [expr $ICMP_receivedNops + [$application_($i) set ICMP_receivedNops_]]

		set SLP_totalQueries [expr $SLP_totalQueries + [$application_($i) set SLP_totalQueries_]]
		set SLP_successfulQueries [expr $SLP_successfulQueries + [$application_($i) set SLP_successfulQueries_]]

		set SERVICE_totalDataRequests [expr $SERVICE_totalDataRequests + [$application_($i) set SERVICE_totalDataRequests_]]
		set SERVICE_successfulDataRequests [expr $SERVICE_successfulDataRequests + [$application_($i) set SERVICE_successfulDataRequests_]]
		set SERVICE_totalSuccessfulDataRequestDelay [expr $SERVICE_totalSuccessfulDataRequestDelay + [$application_($i) set SERVICE_totalSuccessfulDataRequestDelay_]]
		set SERVICE_totalDataUpdates [expr $SERVICE_totalDataUpdates + [$application_($i) set SERVICE_totalDataUpdates_]]
		set SERVICE_successfulDataUpdates [expr $SERVICE_successfulDataUpdates + [$application_($i) set SERVICE_successfulDataUpdates_]]
		set SERVICE_totalStreamRequests [expr $SERVICE_totalStreamRequests + [$application_($i) set SERVICE_totalStreamRequests_]]
		set SERVICE_successfulStreamRequests [expr $SERVICE_successfulStreamRequests + [$application_($i) set SERVICE_successfulStreamRequests_]]
		set SERVICE_expectedStreamPackets [expr $SERVICE_expectedStreamPackets + [$application_($i) set SERVICE_expectedStreamPackets_]]
		set SERVICE_receivedStreamPackets [expr $SERVICE_receivedStreamPackets + [$application_($i) set SERVICE_receivedStreamPackets_]]
		set SERVICE_totalRequests [expr $SERVICE_totalRequests + [$application_($i) set SERVICE_totalRequests_]]
		set SERVICE_successfulRequests [expr $SERVICE_successfulRequests + [$application_($i) set SERVICE_successfulRequests_]]
		set SERVICE_transmittedBytes [expr $SERVICE_transmittedBytes + [$application_($i) set SERVICE_transmittedBytes_]]
		set SERVICE_receivedBytes [expr $SERVICE_receivedBytes + [$application_($i) set SERVICE_receivedBytes_]]
		set SERVICE_currentTransmittedDatarate [expr $SERVICE_currentTransmittedDatarate + [$application_($i) set SERVICE_transmittedDatarate_]]
		set SERVICE_currentReceivedDatarate [expr $SERVICE_currentReceivedDatarate + [$application_($i) set SERVICE_receivedDatarate_]]
		for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
			set SERVICE_requestPacketsPerHopCount($hopCount) [expr $SERVICE_requestPacketsPerHopCount($hopCount) + [$application_($i) set [format "SERVICE_requestPacketsPerHopCount_%d_" $hopCount]]]
		}


		set SERVICE_currentlyHostedServices [expr $SERVICE_currentlyHostedServices + [$application_($i) set SERVICE_currentlyHostedServices_]]
		set SERVICE_averageHostedServices [expr $SERVICE_averageHostedServices + [$application_($i) set SERVICE_averageHostedServices_]]
		if {$SERVICE_currentlyHostedServices > 0} then {
			set SERVICE_availableUntilTime $current_time
		}
		set SERVICE_syncOverhead_absolute [expr $SERVICE_syncOverhead_absolute + [$application_($i) set SERVICE_syncOverheadBytes_]]
		set SERVICE_totalReplications [expr $SERVICE_totalReplications + [$application_($i) set SERVICE_totalReplications_]]
		set SERVICE_successfulReplications [expr $SERVICE_successfulReplications + [$application_($i) set SERVICE_successfulReplications_]]
		set SERVICE_totalMigrations [expr $SERVICE_totalMigrations + [$application_($i) set SERVICE_totalMigrations_]]
		set SERVICE_successfulMigrations [expr $SERVICE_successfulMigrations + [$application_($i) set SERVICE_successfulMigrations_]]

		if {[$application_($i) set PLACEMENT_currentServiceCost_] != 0} {
			if {$PLACEMENT_currentServiceCost == 0} {
				set PLACEMENT_currentServiceCost [$application_($i) set PLACEMENT_currentServiceCost_]
			} else {
				puts [format "ERROR: PLACEMENT_currentServiceCost already set while processing node #%d. This indicates that two coordinators are running at the same time, which shouldn't happen." $i]
				exit 2
			}
		}
		if {[$application_($i) set PLACEMENT_currentlyOptimalServiceCost_] != 0} {
			if {$PLACEMENT_currentlyOptimalServiceCost == 0} {
				set PLACEMENT_currentlyOptimalServiceCost [$application_($i) set PLACEMENT_currentlyOptimalServiceCost_]
			} else {
				puts [format "ERROR: PLACEMENT_currentlyOptimalServiceCost already set while processing node #%d. This indicates that two coordinators are running at the same time, which shouldn't happen." $i]
				exit 2
			}
		}
		if {[$application_($i) set PLACEMENT_currentlyOptimalNumberOfHosts_] != 0} {
			if {$PLACEMENT_currentlyOptimalNumberOfHosts == 0} {
				set PLACEMENT_currentlyOptimalNumberOfHosts [$application_($i) set PLACEMENT_currentlyOptimalNumberOfHosts_]
			} else {
				puts [format "ERROR: PLACEMENT_currentlyOptimalNumberOfHosts already set while processing node #%d. This indicates that two coordinators are running at the same time, which shouldn't happen." $i]
				exit 2
			}
		}
		if {[$application_($i) set PLACEMENT_currentlyEstimatedAdaptationCost_] != 0} {
			if {$PLACEMENT_currentlyEstimatedAdaptationCost == 0} {
				set PLACEMENT_currentlyEstimatedAdaptationCost [$application_($i) set PLACEMENT_currentlyEstimatedAdaptationCost_]
			} else {
				puts [format "ERROR: PLACEMENT_currentlyEstimatedAdaptationCost already set while processing node #%d. This indicates that two coordinators are running at the same time, which shouldn't happen." $i]
				exit 2
			}
		}
		set PLACEMENT_totalAdaptations [expr $PLACEMENT_totalAdaptations + [$application_($i) set PLACEMENT_totalAdaptations_]]
		set PLACEMENT_successfulAdaptations [expr $PLACEMENT_successfulAdaptations + [$application_($i) set PLACEMENT_successfulAdaptations_]]

		if {$printReadableOuput} {	;# Only check this at the very end.
			if {([$application_($i) set SERVICE_totalRequests_] > 0) && ([$application_($i) set SERVICE_successfulRequests_] == 0) && ([$application_($i) set SLP_totalQueries_] > 0) && ([$application_($i) set SLP_successfulQueries_] == 0)} {
				puts [format "ERROR: %d service request(s) while processing node #%d, but none of them was successful (%d/%d SLP recall). This indicates that the nodes were placed in such a way that the network is disconnected." [$application_($i) set SERVICE_totalRequests_] $i [$application_($i) set SLP_successfulQueries_] [$application_($i) set SLP_totalQueries_]]
				exit 2
			}
		}
	}

	set averageNodeDegree [expr $averageNodeDegree / double($val(nn))]
	set averageNodeDistance [expr $averageNodeDistance / double($val(nn))]
	set totalUnicastPacketsWithHopCount	0
	for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
		set averageUnicastHopCount [expr $averageUnicastHopCount + ($unicastPacketsPerHopCount($hopCount) * $hopCount)]
		set totalUnicastPacketsWithHopCount [expr $totalUnicastPacketsWithHopCount + $unicastPacketsPerHopCount($hopCount)]
		if {$unicastPacketsPerHopCount($hopCount) > 0} then {
			set maximalUnicastHopCount $hopCount
		}
	}
	if {$totalUnicastPacketsWithHopCount > 0} then {
		set averageUnicastHopCount [expr $averageUnicastHopCount / double($totalUnicastPacketsWithHopCount)]
	}
	set unicastHopCountHistogram_names [format "unicastPacketsPerHopCount_%d" 0]
	set unicastHopCountHistogram_values $unicastPacketsPerHopCount(0)
	for {set hopCount 1} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
		append unicastHopCountHistogram_names "," [format "unicastPacketsPerHopCount_%d" $hopCount]
		append unicastHopCountHistogram_values "," $unicastPacketsPerHopCount($hopCount)
	}
	if {$transmittedUnicastPackets > 0} then {
		set unicastPDR [expr double($receivedUnicastPackets) / double($transmittedUnicastPackets)]
	}
	if {$transmittedBroadcastPackets > 0} then {
		set broadcastPDR [expr double($receivedBroadcastPackets) / double($transmittedBroadcastPackets)]
	}
	set averageTransmittedDatarate [expr double($transmittedBytes) / double($total_duration)]
	set averageReceivedDatarate [expr double($receivedBytes) / double($total_duration)]
	set overhead_wrt_serviceTraffic_absolute [expr $transmittedBytes - $SERVICE_transmittedBytes]
	if {$transmittedBytes > 0} then {
		set overhead_wrt_serviceTraffic_relative [expr double($overhead_wrt_serviceTraffic_absolute) / double($transmittedBytes)]
	}
	if {$MAC_transmittedBytes > 0} then {
		set networkMappingOverhead_relative [expr double($networkMappingOverhead_absolute) / double($MAC_transmittedBytes)]
	}

	set MAC_averageTransmittedDatarate [expr double($MAC_transmittedBytes) / double($total_duration)]
	set MAC_averageReceivedDatarate [expr double($MAC_receivedBytes) / double($total_duration)]
	set MAC_overhead_wrt_serviceTraffic_absolute [expr $MAC_transmittedBytes - $SERVICE_transmittedBytes]
	if {$MAC_transmittedBytes > 0} then {
		set MAC_overhead_wrt_serviceTraffic_relative [expr double($MAC_overhead_wrt_serviceTraffic_absolute) / double($MAC_transmittedBytes)]
	}

	if {$DYMO_transmittedUnicastPackets > 0} then {
		set DYMO_unicastPdr [expr double($DYMO_receivedUnicastPackets) / double($DYMO_transmittedUnicastPackets)]
	}
	if {$DYMO_transmittedBroadcastPackets > 0} then {
		set DYMO_broadcastPdr [expr double($DYMO_receivedBroadcastPackets) / double($DYMO_transmittedBroadcastPackets)]
	}

	if {$RUDP_totalTransmissions > 0} then {
		set RUDP_pdr [expr double($RUDP_successfulTransmissions) / double($RUDP_totalTransmissions)]
	}

	if {$ICMP_transmittedNops > 0} then {
		set ICMP_nopPdr [expr double($ICMP_receivedNops) / double($ICMP_transmittedNops)]
	}

	if {$SLP_totalQueries > 0} then {
		set SLP_recall [expr double($SLP_successfulQueries) / double($SLP_totalQueries)]
	}

	if {$SERVICE_totalDataRequests > 0} then {
		set SERVICE_dataRequestRecall [expr double($SERVICE_successfulDataRequests) / double($SERVICE_totalDataRequests)]
	}
	if {$SERVICE_successfulDataRequests > 0} then {
		set SERVICE_averageSuccessfulDataRequestDelay [expr double($SERVICE_totalSuccessfulDataRequestDelay) / double($SERVICE_successfulDataRequests)]
	}
	if {$SERVICE_totalDataUpdates > 0} then {
		set SERVICE_dataUpdateRecall [expr double($SERVICE_successfulDataUpdates) / double($SERVICE_totalDataUpdates)]
	}
	if {$SERVICE_totalStreamRequests > 0} then {
		set SERVICE_streamRequestRecall [expr double($SERVICE_successfulStreamRequests) / double($SERVICE_totalStreamRequests)]
	}
	if {$SERVICE_expectedStreamPackets > 0} then {
		set SERVICE_streamPacketRecall [expr double($SERVICE_receivedStreamPackets) / double($SERVICE_expectedStreamPackets)]
	}
	if {$SERVICE_totalRequests > 0} then {
		set SERVICE_requestRecall [expr double($SERVICE_successfulRequests) / double($SERVICE_totalRequests)]
	}
	set SERVICE_averageTransmittedDatarate [expr double($SERVICE_transmittedBytes) / double($total_duration)]
	set SERVICE_averageReceivedDatarate [expr double($SERVICE_receivedBytes) / double($total_duration)]
	set SERVICE_totalRequestPacketsWithHopCount	0
	set SERVICE_maximalRequestPacketsPerHopCount	0
	for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
		set SERVICE_averageRequestHopCount [expr $SERVICE_averageRequestHopCount + ($SERVICE_requestPacketsPerHopCount($hopCount) * $hopCount)]
		set SERVICE_totalRequestPacketsWithHopCount [expr $SERVICE_totalRequestPacketsWithHopCount + $SERVICE_requestPacketsPerHopCount($hopCount)]
		if {$SERVICE_requestPacketsPerHopCount($hopCount) > 0} then {
			set SERVICE_maximalRequestHopCount $hopCount
		}
	}
	if {$SERVICE_totalRequestPacketsWithHopCount > 0} then {
		set SERVICE_averageRequestHopCount [expr $SERVICE_averageRequestHopCount / double($SERVICE_totalRequestPacketsWithHopCount)]
	}
	set SERVICE_requestHopCountHistogram_names [format "SERVICE_requestPacketsPerHopCount_%d" 0]
	set SERVICE_requestHopCountHistogram_values $SERVICE_requestPacketsPerHopCount(0)
	for {set hopCount 1} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
		append SERVICE_requestHopCountHistogram_names "," [format "SERVICE_requestPacketsPerHopCount_%d" $hopCount]
		append SERVICE_requestHopCountHistogram_values "," $SERVICE_requestPacketsPerHopCount($hopCount)
	}

	if {$SERVICE_transmittedBytes > 0} then {
		set SERVICE_syncOverhead_relative [expr double($SERVICE_syncOverhead_absolute) / double($SERVICE_transmittedBytes)]
	}
	if {$SERVICE_totalReplications > 0} then {
		set SERVICE_replicationSuccessRate [expr double($SERVICE_successfulReplications) / double($SERVICE_totalReplications)]
	}
	if {$SERVICE_totalMigrations > 0} then {
		set SERVICE_migrationSuccessRate [expr double($SERVICE_successfulMigrations) / double($SERVICE_totalMigrations)]
	}
	if {$PLACEMENT_totalAdaptations > 0} then {
		set PLACEMENT_adaptationSuccessRate [expr double($PLACEMENT_successfulAdaptations) / double($PLACEMENT_totalAdaptations)]
	}



	if {$DEBUG_WARNs >= (1000 * $val(nn))} {
		puts [format "ERROR: Reached %s WARNings in a scenario with %d nodes. This might indicate an error in the implementation." [thousands $DEBUG_WARNs] $val(nn)]
		exit 2
	}
	if {($SLP_totalQueries >= (3 * $val(nn))) && ($SLP_recall < 0.10)} {
		puts [format "ERROR: Reached %s SLP queries while recall remains at %.2f%%. This indicates that the nodes were placed in such a way that the network is disconnected." [thousands $SLP_totalQueries] [expr $SLP_recall * 100.0]]
		exit 2
	}



	if {$printReadableOuput} {
		puts [format "ERRORs:\t\t\t\t\t%d"				$DEBUG_ERRORs]
		puts [format "WARNings:\t\t\t\t%d"				$DEBUG_WARNs]
		puts ""
		puts [format "Nodes:\t\t\t\t\t%d"				$val(nn)]
		puts [format "Area:\t\t\t\t\t%dm x %dm"				$val(x) $val(y)]
#		puts [format "Simulation setup duration:\t\t%.1fs"		$simulation_setup_duration]
#		puts [format "Simulation duration:\t\t\t%.1fs"			$simulation_duration]
		puts [format "Total duration:\t\t\t\t%.1fs (%.1fs + %.1fs)"	$total_duration $simulation_setup_duration $simulation_duration]
		puts [format "Average (mean) node degree:\t\t%.2f"		$averageNodeDegree]
		puts [format "Average (mean) node distance:\t\t%.2f hop(s)"	$averageNodeDistance]
		puts [format "Average (mean) unicast hop count:\t%.2f hop(s)"	$averageUnicastHopCount]
		puts [format "Maximal unicast hop count:\t\t%d hop(s)"		$maximalUnicastHopCount]
		puts [format "Unicast packets by hop counts:\t\t%s"		$unicastHopCountHistogram_values]
#		for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
#			puts [format "Unicast packets with hop count %d:\t%s"	$hopCount [thousands $unicastPacketsPerHopCount($hopCount)]]
#		}
		puts [format "Transmitted unicast packets:\t\t%s"		[thousands $transmittedUnicastPackets]]
		puts [format "Unicast transmission failures:\t\t%s"		[thousands $unicastTransmissionFailures]]
		puts [format "Received unicast packets:\t\t%s"			[thousands $receivedUnicastPackets]]
		puts [format "Unicast packet delivery ratio:\t\t%.2f%%"		[expr $unicastPDR * 100.0]]
		puts [format "Transmitted broadcast packets:\t\t%s"		[thousands $transmittedBroadcastPackets]]
		puts [format "Received broadcast packets:\t\t%s"		[thousands $receivedBroadcastPackets]]
		puts [format "Broadcast packet delivery ratio:\t%.2f%%"		[expr $broadcastPDR * 100.0]]
		puts [format "Total traffic (sent):\t\t\t%s bytes"		[thousands $transmittedBytes]]
		puts [format "Total traffic (received):\t\t%s bytes"		[thousands $receivedBytes]]
		puts [format "Current datarate (sent):\t\t%s bytes/s"		[thousands $currentTransmittedDatarate]]
		puts [format "Current datarate (received):\t\t%s bytes/s"	[thousands $currentReceivedDatarate]]
		puts [format "Average datarate (sent):\t\t%s bytes/s"		[thousands $averageTransmittedDatarate]]
		puts [format "Average datarate (received):\t\t%s bytes/s"	[thousands $averageReceivedDatarate]]
		puts [format "Overhead (wrt. SERVICE, absolute):\t%s bytes"	[thousands $overhead_wrt_serviceTraffic_absolute]]
		puts [format "Overhead (wrt. SERVICE, relative):\t%.2f%%"	[expr $overhead_wrt_serviceTraffic_relative * 100.0]]
		puts [format "Network mapping overhead (absolute):\t%s bytes"	[thousands $networkMappingOverhead_absolute]]
		puts [format "Network mapping overhead (relative):\t%.2f%%"	[expr $networkMappingOverhead_relative * 100.0]]
		puts ""
		puts [format "MAC transmitted packets:\t\t%s"			[thousands $MAC_transmittedPackets]]
		puts [format "MAC transmission failures:\t\t%s"			[thousands $MAC_transmissionFailures]]
		puts [format "MAC received packets:\t\t\t%s"			[thousands $MAC_receivedPackets]]
		puts [format "MAC dropped packets (capture):\t\t%s"		[thousands $MAC_droppedPackets_capture]]
		puts [format "MAC dropped packets (collision):\t%s"		[thousands $MAC_droppedPackets_collision]]
		puts [format "MAC dropped packets (transmit):\t\t%s"		[thousands $MAC_droppedPackets_transmit]]
		puts [format "MAC dropped packets (bit error):\t%s"		[thousands $MAC_droppedPackets_bit_error]]
		puts [format "MAC total traffic (sent):\t\t%s bytes"		[thousands $MAC_transmittedBytes]]
		puts [format "MAC total traffic (received):\t\t%s bytes"	[thousands $MAC_receivedBytes]]
		puts [format "MAC average datarate (sent):\t\t%s bytes/s"	[thousands $MAC_averageTransmittedDatarate]]
		puts [format "MAC average datarate (received):\t%s bytes/s"	[thousands $MAC_averageReceivedDatarate]]
		puts [format "MAC overhead (wrt. SERVICE, absolute):\t%s bytes"	[thousands $MAC_overhead_wrt_serviceTraffic_absolute]]
		puts [format "MAC overhead (wrt. SERVICE, relative):\t%.2f%%"	[expr $MAC_overhead_wrt_serviceTraffic_relative * 100.0]]
		puts ""
		puts [format "DYMO transmitted route requests (RREQ):\t%s"	[thousands $DYMO_transmittedRREQs]]
		puts [format "DYMO transmitted route replies (RREP):\t%s"	[thousands $DYMO_transmittedRREPs]]
		puts [format "DYMO transmitted route errors (RERR):\t%s"	[thousands $DYMO_transmittedRERRs]]
		puts [format "DYMO transmitted unicast packets:\t%s"		[thousands $DYMO_transmittedUnicastPackets]]
		puts [format "DYMO received unicast packets:\t\t%s"		[thousands $DYMO_receivedUnicastPackets]]
		puts [format "DYMO unicast packet delivery ratio:\t%.2f%%"	[expr $DYMO_unicastPdr * 100.0]]
		puts [format "DYMO transmitted broadcast packets:\t%s"		[thousands $DYMO_transmittedBroadcastPackets]]
		puts [format "DYMO received broadcast packets:\t%s"		[thousands $DYMO_receivedBroadcastPackets]]
		puts [format "DYMO broadcast packet delivery ratio:\t%.2f%%"	[expr $DYMO_broadcastPdr * 100.0]]
		puts ""
		puts [format "RUDP total transmissions:\t\t%s"			[thousands $RUDP_totalTransmissions]]
		puts [format "RUDP successful transmissions:\t\t%s"		[thousands $RUDP_successfulTransmissions]]
		puts [format "RUDP packet delivery ratio:\t\t%.2f%%"		[expr $RUDP_pdr * 100.0]]
		puts ""
#		puts [format "ICMP transmitted NOPs:\t\t\t%s"			[thousands $ICMP_transmittedNops]]
#		puts [format "ICMP received NOPs:\t\t\t%s"			[thousands $ICMP_receivedNops]]
#		puts [format "ICMP NOP packet delivery ratio:\t\t%.2f%%"	[expr $ICMP_nopPdr * 100.0]]
#		if {$ICMP_transmittedNops > 0} then {
#			for {set i 0} {$i < $val(nn)} {incr i} {
#				puts [format "ICMP NOP packet delivery ratio at node #%d (%.2f, %.2f, %.2f):\t%.2f%%" $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_] [expr double([$application_($i) set ICMP_receivedNops_]) / double($ICMP_transmittedNops) * 100.0]]
#			}
#		}
#		puts ""
		puts [format "SLP total queries:\t\t\t%s"			[thousands $SLP_totalQueries]]
		puts [format "SLP successful queries:\t\t\t%s"			[thousands $SLP_successfulQueries]]
		puts [format "SLP recall:\t\t\t\t%.2f%%"			[expr $SLP_recall * 100.0]]
		puts ""
		puts [format "SERVICE total data requests:\t\t%s"		[thousands $SERVICE_totalDataRequests]]
		puts [format "SERVICE successful data requests:\t%s"		[thousands $SERVICE_successfulDataRequests]]
		puts [format "SERVICE data request recall:\t\t%.2f%%"		[expr $SERVICE_dataRequestRecall * 100.0]]
		puts [format "SERVICE avg. succ. data request delay:\t%.4fs"	$SERVICE_averageSuccessfulDataRequestDelay]
		puts [format "SERVICE total data updates:\t\t%s"		[thousands $SERVICE_totalDataUpdates]]
		puts [format "SERVICE successful data updates:\t%s"		[thousands $SERVICE_successfulDataUpdates]]
		puts [format "SERVICE data update recall:\t\t%.2f%%"		[expr $SERVICE_dataUpdateRecall * 100.0]]
#		puts [format "SERVICE total stream requests:\t\t%s"		[thousands $SERVICE_totalStreamRequests]]
#		puts [format "SERVICE successful stream requests:\t%s"		[thousands $SERVICE_successfulStreamRequests]]
#		puts [format "SERVICE stream request recall:\t\t%.2f%%"		[expr $SERVICE_streamRequestRecall * 100.0]]
#		puts [format "SERVICE expected stream packets:\t%s"		[thousands $SERVICE_expectedStreamPackets]]
#		puts [format "SERVICE received stream packets:\t%s"		[thousands $SERVICE_receivedStreamPackets]]
#		puts [format "SERVICE stream packet recall:\t\t%.2f%%"		[expr $SERVICE_streamPacketRecall * 100.0]]
		puts [format "SERVICE total requests:\t\t\t%s"			[thousands $SERVICE_totalRequests]]
		puts [format "SERVICE successful requests:\t\t%s"		[thousands $SERVICE_successfulRequests]]
		puts [format "SERVICE request recall:\t\t\t%.2f%%"		[expr $SERVICE_requestRecall * 100.0]]
		puts [format "SERVICE total traffic (sent):\t\t%s bytes"	[thousands $SERVICE_transmittedBytes]]
		puts [format "SERVICE total traffic (received):\t%s bytes"	[thousands $SERVICE_receivedBytes]]
		puts [format "SERVICE current datarate (sent):\t%s bytes/s"	[thousands $SERVICE_currentTransmittedDatarate]]
		puts [format "SERVICE current datarate (received):\t%s bytes/s"	[thousands $SERVICE_currentReceivedDatarate]]
		puts [format "SERVICE average datarate (sent):\t%s bytes/s"	[thousands $SERVICE_averageTransmittedDatarate]]
		puts [format "SERVICE average datarate (received):\t%s bytes/s"	[thousands $SERVICE_averageReceivedDatarate]]
		puts [format "SERVICE average request hop count:\t%.2f hop(s)"	$SERVICE_averageRequestHopCount]
		puts [format "SERVICE maximal request hop count:\t%d hop(s)"	$SERVICE_maximalRequestHopCount]
		puts [format "SERVICE requests by hop counts:\t\t%s"		$SERVICE_requestHopCountHistogram_values]
#		for {set hopCount 0} {$hopCount <= $maximal_network_diameter} {incr hopCount} {
#			puts [format "SERVICE requests with hop count %d:\t%s"	$hopCount [thousands $SERVICE_requestPacketsPerHopCount($hopCount)]]
#		}
		puts ""
		puts [format "SERVICE currently hosted services:\t%s"		[thousands $SERVICE_currentlyHostedServices]]
		puts [format "SERVICE average hosted services:\t%s"		[thousands $SERVICE_averageHostedServices]]
		puts [format "SERVICE globally available until:\t%.2fs"		$SERVICE_availableUntilTime]
		puts [format "SERVICE interhost sync overhead (abs.):\t%s bytes"	[thousands $SERVICE_syncOverhead_absolute]]
		puts [format "SERVICE interhost sync overhead (rel.):\t%.2f%%"	[expr $SERVICE_syncOverhead_relative * 100.0]]
		puts [format "SERVICE total replications:\t\t%s"		[thousands $SERVICE_totalReplications]]
		puts [format "SERVICE successful replications:\t%s"		[thousands $SERVICE_successfulReplications]]
		puts [format "SERVICE replications success rate:\t%.2f%%"	[expr $SERVICE_replicationSuccessRate * 100.0]]
		puts [format "SERVICE total migrations:\t\t%s"			[thousands $SERVICE_totalMigrations]]
		puts [format "SERVICE successful migrations:\t\t%s"		[thousands $SERVICE_successfulMigrations]]
		puts [format "SERVICE migrations success rate:\t%.2f%%"		[expr $SERVICE_migrationSuccessRate * 100.0]]
		puts ""
		puts [format "PLACEMENT current service cost:\t\t%s"		[thousands $PLACEMENT_currentServiceCost]]
		puts [format "PLACEMENT curr. optimal service cost:\t%s"	[thousands $PLACEMENT_currentlyOptimalServiceCost]]
		puts [format "PLACEMENT curr. opt. number of hosts:\t%s"	[thousands $PLACEMENT_currentlyOptimalNumberOfHosts]]
		puts [format "PLACEMENT curr. est. adaptation cost:\t%s"	[thousands $PLACEMENT_currentlyEstimatedAdaptationCost]]
		puts [format "PLACEMENT total adaptations:\t\t%s"		[thousands $PLACEMENT_totalAdaptations]]
		puts [format "PLACEMENT successful adaptations:\t%s"		[thousands $PLACEMENT_successfulAdaptations]]
		puts [format "PLACEMENT adaptations success rate:\t%.2f%%"	[expr $PLACEMENT_adaptationSuccessRate * 100.0]]
	}

	if {$printParseableHeader} {
		puts ""
		puts "###"
		puts "current_time,nodes,area_x,area_y,setup_duration,run_duration,total_duration,averageNodeDegree,averageNodeDistance,averageUnicastHopCount,maximalUnicastHopCount,$unicastHopCountHistogram_names,transmittedUnicastPackets,unicastTransmissionFailures,receivedUnicastPackets,unicastPDR,transmittedBroadcastPackets,receivedBroadcastPackets,broadcastPDR,transmittedBytes,receivedBytes,currentTransmittedDatarate,currentReceivedDatarate,averageTransmittedDatarate,averageReceivedDatarate,overhead_wrt_serviceTraffic_absolute,overhead_wrt_serviceTraffic_relative,networkMappingOverhead_absolute,MAC_transmittedPackets,MAC_transmissionFailures,MAC_receivedPackets,MAC_droppedPackets_capture,MAC_droppedPackets_collision,MAC_droppedPackets_transmit,MAC_droppedPackets_bit_error,MAC_transmittedBytes,MAC_receivedBytes,MAC_averageTransmittedDatarate,MAC_averageReceivedDatarate,MAC_overhead_wrt_serviceTraffic_absolute,MAC_overhead_wrt_serviceTraffic_relative,DYMO_transmittedRREQs,DYMO_transmittedRREPs,DYMO_transmittedRERRs,DYMO_transmittedUnicastPackets,DYMO_receivedUnicastPackets,DYMO_unicastPdr,DYMO_transmittedBroadcastPackets,DYMO_receivedBroadcastPackets,DYMO_broadcastPdr,RUDP_totalTransmissions,RUDP_successfulTransmissions,RUDP_pdr,ICMP_transmittedNops,ICMP_receivedNops,ICMP_nopPdr,SLP_totalQueries,SLP_successfulQueries,SLP_recall,SERVICE_totalDataRequests,SERVICE_successfulDataRequests,SERVICE_dataRequestRecall,SERVICE_averageSuccessfulDataRequestDelay,SERVICE_totalDataUpdates,SERVICE_successfulDataUpdates,SERVICE_dataUpdateRecall,SERVICE_totalStreamRequests,SERVICE_successfulStreamRequests,SERVICE_streamRequestRecall,SERVICE_expectedStreamPackets,SERVICE_receivedStreamPackets,SERVICE_streamPacketRecall,SERVICE_totalRequests,SERVICE_successfulRequests,SERVICE_requestRecall,SERVICE_transmittedBytes,SERVICE_receivedBytes,SERVICE_currentTransmittedDatarate,SERVICE_currentReceivedDatarate,SERVICE_averageTransmittedDatarate,SERVICE_averageReceivedDatarate,SERVICE_averageRequestHopCount,SERVICE_maximalRequestHopCount,$SERVICE_requestHopCountHistogram_names,SERVICE_currentlyHostedServices,SERVICE_averageHostedServices,SERVICE_availableUntilTime,SERVICE_syncOverhead_absolute,SERVICE_syncOverhead_relative,SERVICE_totalReplications,SERVICE_successfulReplications,SERVICE_replicationSuccessRate,SERVICE_totalMigrations,SERVICE_successfulMigrations,SERVICE_migrationSuccessRate,PLACEMENT_currentServiceCost,PLACEMENT_currentlyOptimalServiceCost,PLACEMENT_currentlyOptimalNumberOfHosts,PLACEMENT_currentlyEstimatedAdaptationCost,PLACEMENT_totalAdaptations,PLACEMENT_successfulAdaptations,PLACEMENT_adaptationSuccessRate"
	}

	if {$printParseableResults} {
		puts "$current_time,$val(nn),$val(x),$val(y),$simulation_setup_duration,$simulation_duration,$total_duration,$averageNodeDegree,$averageNodeDistance,$averageUnicastHopCount,$maximalUnicastHopCount,$unicastHopCountHistogram_values,$transmittedUnicastPackets,$unicastTransmissionFailures,$receivedUnicastPackets,$unicastPDR,$transmittedBroadcastPackets,$receivedBroadcastPackets,$broadcastPDR,$transmittedBytes,$receivedBytes,$currentTransmittedDatarate,$currentReceivedDatarate,$averageTransmittedDatarate,$averageReceivedDatarate,$overhead_wrt_serviceTraffic_absolute,$overhead_wrt_serviceTraffic_relative,$networkMappingOverhead_absolute,$MAC_transmittedPackets,$MAC_transmissionFailures,$MAC_receivedPackets,$MAC_droppedPackets_capture,$MAC_droppedPackets_collision,$MAC_droppedPackets_transmit,$MAC_droppedPackets_bit_error,$MAC_transmittedBytes,$MAC_receivedBytes,$MAC_averageTransmittedDatarate,$MAC_averageReceivedDatarate,$MAC_overhead_wrt_serviceTraffic_absolute,$MAC_overhead_wrt_serviceTraffic_relative,$DYMO_transmittedRREQs,$DYMO_transmittedRREPs,$DYMO_transmittedRERRs,$DYMO_transmittedUnicastPackets,$DYMO_receivedUnicastPackets,$DYMO_unicastPdr,$DYMO_transmittedBroadcastPackets,$DYMO_receivedBroadcastPackets,$DYMO_broadcastPdr,$RUDP_totalTransmissions,$RUDP_successfulTransmissions,$RUDP_pdr,$ICMP_transmittedNops,$ICMP_receivedNops,$ICMP_nopPdr,$SLP_totalQueries,$SLP_successfulQueries,$SLP_recall,$SERVICE_totalDataRequests,$SERVICE_successfulDataRequests,$SERVICE_dataRequestRecall,$SERVICE_averageSuccessfulDataRequestDelay,$SERVICE_totalDataUpdates,$SERVICE_successfulDataUpdates,$SERVICE_dataUpdateRecall,$SERVICE_totalStreamRequests,$SERVICE_successfulStreamRequests,$SERVICE_streamRequestRecall,$SERVICE_expectedStreamPackets,$SERVICE_receivedStreamPackets,$SERVICE_streamPacketRecall,$SERVICE_totalRequests,$SERVICE_successfulRequests,$SERVICE_requestRecall,$SERVICE_transmittedBytes,$SERVICE_receivedBytes,$SERVICE_currentTransmittedDatarate,$SERVICE_currentReceivedDatarate,$SERVICE_averageTransmittedDatarate,$SERVICE_averageReceivedDatarate,$SERVICE_averageRequestHopCount,$SERVICE_maximalRequestHopCount,$SERVICE_requestHopCountHistogram_values,$SERVICE_currentlyHostedServices,$SERVICE_averageHostedServices,$SERVICE_availableUntilTime,$SERVICE_syncOverhead_absolute,$SERVICE_syncOverhead_relative,$SERVICE_totalReplications,$SERVICE_successfulReplications,$SERVICE_replicationSuccessRate,$SERVICE_totalMigrations,$SERVICE_successfulMigrations,$SERVICE_migrationSuccessRate,$PLACEMENT_currentServiceCost,$PLACEMENT_currentlyOptimalServiceCost,$PLACEMENT_currentlyOptimalNumberOfHosts,$PLACEMENT_currentlyEstimatedAdaptationCost,$PLACEMENT_totalAdaptations,$PLACEMENT_successfulAdaptations,$PLACEMENT_adaptationSuccessRate"
	}
}
