##
#
# Configuration options.
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: config.tcl 3178 2010-01-05 16:52:35Z wittenbu $
#
##



# Configure network protocol stack
set val(engmodel)	EnergyModel				;# energy model
set val(txPower)        0.0;					;# transmitting power (in W)
set val(rxPower)        0.0;					;# receiving power (in W)
set val(idlePower)      0.0;					;# idle power (in W)
set val(initeng)	1.0					;# initial energy (in J)
set val(chan)		Channel/WirelessChannel			;# channel type
#set val(prop)		Propagation/FreeSpace			;# radio-propagation model
set val(prop)		Propagation/TwoRayGround		;# radio-propagation model
#set val(prop)		Propagation/Shadowing			;# radio-propagation model
set val(ant)		Antenna/OmniAntenna			;# antenna model
set val(netif)		Phy/WirelessPhy				;# network interface type
set val(mac)		Mac/802_11				;# MAC type
set val(ll)		LL					;# link layer type
set val(ifq)		Queue/DropTail/PriQueue			;# interface queue type (using default value)
set val(ifqlen)		50					;# max packet in interface queue (using default value)
set val(rp)		DumbAgent				;# routing protocol (none)
#set val(rp)		AODV					;# routing protocol (AODV, ns-2 implementation)
#set val(rp)		AODVUU					;# routing protocol (AODV, Uppsala University implementation)
#set val(rp)		DYMOUM					;# routing protocol (DYMO, Universidad de Murcia implementation)


# Adjust parameters to match 802.11b as implemented in an ORiNOCO 11b PC card.
#
# The resulting transmission radius is 50m.
#
# N.B. Our values are only based on the datasheet as the other two sources
# appear to be erroneous.
#
# Sources:
# Proxim Corporation. ORiNOCO 11b Client PC Card. Datasheet, 2003
#   (http://www.proxim.com/learn/library/datasheets/11bpccard.pdf)
# Wu Xiuchao. Simulate 802.11b Channel within NS2. Technical Report, National University of Singapore, 2004.
#   (http://www.comp.nus.edu.sg/~wuxiucha/research/reactive/publication/Simulate80211ChannelWithNS2.pdf)
# Bjrn Wiberg. RxThreshold and CSThreshold values!. Mail on ns-users mailing list, 9 July 2002
#   (http://mailman.isi.edu/pipermail/ns-users/2002-July/024110.html)
# dBm to mW Calculator (http://www.aubraux.com/design/dbm-to-milli-watts-calculator.php)

#Antenna/OmniAntenna	set Gt_		1.0		;# (default)
#Antenna/OmniAntenna	set Gr_		1.0		;# (default)
#Phy/WirelessPhy	set CPThresh_	10.0		;# (default)
Phy/WirelessPhy		set CSThresh_	6.30957e-12	;# carrier sense threshold according to datasheet
							;# (-82 dBm = 6.3095734448019e-12 W)
Phy/WirelessPhy		set RXThresh_	1.17974e-09	;# receive power threshold calculated with
							;# ns/indep-utils/propagation/threshold using datasheet values for
							;# semi-open scenario for shadowing propagation model
							;# (50 m -> 1.17974e-09 W)
Phy/WirelessPhy		set bandwidth_	11Mb		;# bandwidth (11 Mbit/s)
Phy/WirelessPhy		set Pt_		0.031622777	;# transmit power according to datasheet (15dB m = 0.031622777 W)
Phy/WirelessPhy		set freq_	2.472e9		;# 802.11b, channel 13 (2.472G Hz)
#Phy/WirelessPhy	set L_		1.0		;# (default)
Mac/802_11		set basicRate_	1Mb		;# data rate for control frames (1 Mbit/s)
Mac/802_11		set dataRate_	11Mb		;# data rate for data frames (11 Mbit/s)

# Disable MAC 802.11 RTS/CTS packets below this threshold.
Mac/802_11		set RTSThreshold_	5000	;# bytes

# set DumbAgent to pass all packets to the upper layer modules
Agent/DumbAgent		set promiscuousMode_	true

# Transport agent
Agent/MessagePassing	set packetSize_			4096	;# allow for larger packets to be sent (default is 1500 bytes)
Agent/MessagePassing	set processPackets_		true	;# enable processing of packets by application
Agent/MessagePassing	set notifyOnXmitFailure_	true	;# notify application on failed packet transmissions


# Maximal network diameter (in hops).
# NOTE: This value must match the declaration of C_MAXIMAL_NETWORK_DIAMETER
# in config.h.
set maximal_network_diameter	20

# The spacing distance for nodes in a grid configuration.
set grid_spacing_distance	30
