##
#
# Placement evaluation script.
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# Author: Georg Wittenburg
#
# $Id: placement_evaluation.tcl 3178 2010-01-05 16:52:35Z wittenbu $
#
##



source common/config.tcl

# Number of mobile nodes.
set val(nn)	%%nodes%%

# Size of topography (m) is initialized while setting up nodes.
#set val(x)	1000
set val(x)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]
#set val(y)	1000
set val(y)	[expr int(ceil(sqrt($val(nn)))) * $grid_spacing_distance]

# Simulation setup duration (s).
#set simulation_setup_duration	0.0	;# 0 minutes
#set simulation_setup_duration	60.0	;# 1 minute
set simulation_setup_duration	300.0	;# 5 minutes
#set simulation_setup_duration	900.0	;# 15 minutes
#set simulation_setup_duration	3600.0	;# 1 hour

# Simulation duration (s).
#set simulation_duration	60.0	;# 1 minute
#set simulation_duration	300.0	;# 5 minutes
set simulation_duration		900.0	;# 15 minutes
#set simulation_duration	3600.0	;# 1 hour



source common/header.tcl

# Create, initialize and connect nodes, agents and applications.
for {set i 0} {$i < $val(nn)} {incr i} {
	set node_($i) [$ns_ node]
#	$node_($i) set X_ $i
	$node_($i) set X_ [$rng_ uniform 0 $val(x)]
#	$node_($i) set X_ [expr ($i % int(sqrt($val(nn)))) * $grid_spacing_distance]
#	$node_($i) set Y_ 0
	$node_($i) set Y_ [$rng_ uniform 0 $val(y)]
#	$node_($i) set Y_ [expr ($i / int(sqrt($val(nn)))) * $grid_spacing_distance]
	$node_($i) set Z_ 0
	$node_($i) random-motion 0		;# Disable random motion
	$god_ new_node $node_($i)
	$ns_ initial_node_pos $node_($i) 10
	puts [format "Placing node #%d at (%.2f, %.2f, %.2f)..." $i [$node_($i) set X_] [$node_($i) set Y_] [$node_($i) set Z_]]

	set agent_($i) [new Agent/MessagePassing]
	set application_($i) [new Application/C]
	$node_($i) attach $agent_($i)
	$application_($i) attach-agent $agent_($i)
	$application_($i) seed heuristic 0	;# Enable heuristic random seed in application

	set start_time($i) [$rng_ uniform $start_phase_begin $start_phase_end]
#	set stop_time($i) [$rng_ uniform $stop_phase_begin $stop_phase_end]
	set stop_time($i) $stop_phase_begin
	$ns_ at $start_time($i) "$application_($i) start"
	$ns_ at $stop_time($i) "$application_($i) stop"
	$ns_ at $finalize_phase_begin "$application_($i) finalize"
	$ns_ at $simulation_end "$application_($i) SERVICE_set_placement_algorithm none"
}
#puts [format "Center node is node #%d at (%.2f, %.2f, %.2f)." $grid_center_node [$node_($grid_center_node) set X_] [$node_($grid_center_node) set Y_] [$node_($grid_center_node) set Z_]]



#
# Service setup.
#

# Service requests per node per second.
set request_frequency %%request_frequency%%

# Active node ratio (0.0 -> all nodes quiet; 1.0 -> all nodes active).
set active_node_ratio %%activity_factor%%

# Synchronization ratio (0.0 -> no synchronization traffic; 1.0 -> synchronization of every request).
set sync_ratio %%sync_ratio%%

# Node to host the first service instance.
#set initial_service_host 0
#set initial_service_host $grid_center_node
set initial_service_host [$rng_ integer $val(nn)]

# Start initial service instance of "srvA".
$ns_ at 0 "$application_($initial_service_host) SERVICE_create_service srvA %%service_size%% $sync_ratio"
#$ns_ at 1 "$application_($initial_service_host) SERVICE_is_local_service srvA"

# Start placement algorithm.
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA none"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA simple_heuristic"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA graph_cost_single_instance"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA graph_cost_multiple_instances"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA linkpull_migration"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA peerpull_migration"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA topocenter_migration"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA tree-topology_migration"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA asg_simple"
#$ns_ at 0 "$application_($initial_service_host) SERVICE_set_placement_algorithm srvA asg_event_flow_tree"



#
# Placement evaluation.
#

# Have all active nodes in the network request or update data of "srvA".
for {set i 0} {$i < $val(nn)} {incr i} {
	if {[$rng_ uniform 0.0 1.0] <= $active_node_ratio} {
		for {set t $start_time($i)} {$t < [expr $stop_time($i) - (1.0 / $request_frequency)]} {set t [expr $t + (1.0 / $request_frequency)]} {
			set t_rand [$rng_ uniform $t [expr $t + (1.0 / $request_frequency)]]
			if {[$rng_ uniform 0.0 1.0] <= $sync_ratio} {
				$ns_ at $t_rand "$application_($i) SERVICE_update_data srvA 0x0000 1024"
				#puts [format "Scheduling service update from node #%d at %.4f..." $i $t_rand]
			} else {
				$ns_ at $t_rand "$application_($i) SERVICE_request_data srvA 0x0000 1024"
				#puts [format "Scheduling service request from node #%d at %.4f..." $i $t_rand]
			}
		}
	}
}



# Start simulation.
puts "Starting Simulation..."
$ns_ run
