/**
 * @file dymo_vars.h
 *
 * @brief Dynamic MANET On-demand (DYMO) routing protocol (variables).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: dymo_vars.h 3370 2010-02-28 17:20:17Z gwitten $
 */

#ifndef DYMO_VARS_H
#define DYMO_VARS_H

#include "../types.h"
#include "dymo_types.h"



/**
 * @brief Debug level for this component.
 */
#define DYMO_DEBUG_LEVEL	C_MSG_WARN



/**
 * @brief Local sequence number for DYMO control messages.
 *
 * 5.1.1. Maintaining A Node's Own Sequence Number
 *
 * DYMO requires that each DYMO router in the network to maintain its own DYMO
 * sequence number (OwnSeqNum), a 16-bit unsigned integer. The circumstances
 * for ThisNode to incrementing its OwnSeqNum are described in Section 5.3.
 */
uint16_t C_DYMO_own_seq_num;

/**
 * @brief DYMO routing table.
 */
list_t* C_DYMO_routing_table;

/**
 * @brief Queue of packets to be routed once a route has been established.
 */
list_t* C_DYMO_packet_queue;

/**
 * @brief Statistics counter of transmitted RREQ packets.
 */
uint32_t C_DYMO_transmitted_RREQs;

/**
 * @brief Statistics counter of transmitted RREP packets.
 */
uint32_t C_DYMO_transmitted_RREPs;

/**
 * @brief Statistics counter of transmitted RERR packets.
 */
uint32_t C_DYMO_transmitted_RERRs;

/**
 * @brief Statistics counter of transmitted unicast packets.
 */
uint32_t C_DYMO_transmitted_unicast_packets;

/**
 * @brief Statistics counter of received unicast packets.
 */
uint32_t C_DYMO_received_unicast_packets;

/**
 * @brief Statistics counter of transmitted broadcast packets.
 */
uint32_t C_DYMO_transmitted_broadcast_packets;

/**
 * @brief Statistics counter of received broadcast packets.
 */
uint32_t C_DYMO_received_broadcast_packets;

#endif	// DYMO_VARS_H
