/**
 * @file placement.h
 *
 * @brief Placement main file (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: placement.h 3424 2010-03-18 14:45:09Z gwitten $
 */

#ifndef PLACEMENT_H
#define PLACEMENT_H

#include "../types.h"
#include "../service/service_types.h"
#include "placement_types.h"
#include "placement_vars.h"



#define C_PLACEMENT_ALGORITHM_NONE				0x0	///< No placement algorithm.
#define C_PLACEMENT_ALGORITHM_SIMPLE_HEURISTIC			0x1	///< Simple heuristic algorithm.
#define C_PLACEMENT_ALGORITHM_GRAPH_COST_SINGLE_INSTANCE	0x2	///< Graph cost single instance algorithm.
#define C_PLACEMENT_ALGORITHM_GRAPH_COST_MULTIPLE_INSTANCES	0x3	///< Graph cost multiple instances algorithm.
#define C_PLACEMENT_ALGORITHM_LINKPULL_MIGRATION		0x4	///< LinkPull migration algorithm.
#define C_PLACEMENT_ALGORITHM_PEERPULL_MIGRATION		0x5	///< PeerPull migration algorithm.
#define C_PLACEMENT_ALGORITHM_TOPOCENTER_MIGRATION		0x6	///< TopoCenter(1) migration algorithm.
#define C_PLACEMENT_ALGORITHM_TREE_TOPOLOGY_MIGRATION		0x7	///< Tree-topology migration algorithm.
#define C_PLACEMENT_ALGORITHM_ASG_SIMPLE			0x8	///< ASG simple algorithm.
#define C_PLACEMENT_ALGORITHM_ASG_EVENT_FLOW_TREE		0x9	///< ASG event flow tree algorithm.



/**
 * @brief Debug level for this component.
 */
#define PLACEMENT_DEBUG_LEVEL		C_MSG_NOTE

#define C_PLACEMENT_TRIGGER_REASON_DATA				0x0	///< Algorithm triggered because a service data packet has been received.
#define C_PLACEMENT_TRIGGER_REASON_STATS			0x1	///< Algorithm triggered because a service stats packet has been received.

// Interface to main module.
void PLACEMENT_init();
void PLACEMENT_start();
void PLACEMENT_stop();
void PLACEMENT_finalize();
void PLACEMENT_recv(const net_header_t* header, const void* payload);
bool PLACEMENT_command(int argc, const char*const* argv);

// Interface provided by this module.
void PLACEMENT_start_algorithm(SERVICE_service_t* service, uint8_t algorithm_id);
void PLACEMENT_stop_algorithm(SERVICE_service_t* service);
void PLACEMENT_trigger_algorithm(SERVICE_service_t* service, uint8_t reason_id, const void* reason_parameter);
size_t PLACEMENT_get_algorithm_data(const SERVICE_service_t* service, char** data);
void PLACEMENT_set_algorithm_data(SERVICE_service_t* service, const char* data, size_t data_size);
bool PLACEMENT_update_algorithm_data(SERVICE_service_t* service, const PLACEMENT_incoming_adaptation_t* adaptation);
void PLACEMENT_report_known_service_instance(SERVICE_service_t* service, C_addr_t host_address);
void PLACEMENT_report_invalid_service_instance(SERVICE_service_t* service, C_addr_t host_address);
list_t* PLACEMENT_get_synchronization_hosts(const SERVICE_service_t* service, bool inter_host_update);
void PLACEMENT_distribute_adaptation(const SERVICE_id_t* service_id, const list_t* commands, const list_t* destination_nodes, callback_function_t callback, PLACEMENT_adaptation_result_t* result);
void PLACEMENT_update_remote_coordinator(const SERVICE_id_t* service_id, C_addr_t destination_node_address, C_addr_t coordinator_address); void PLACEMENT_shutdown_remote_service_host(const SERVICE_id_t* service_id, C_addr_t destination_node_address);
void PLACEMENT_print_command(uint8_t debug_level, uint8_t msg_type, const void* item);

#endif	// PLACEMENT_H
