/**
 * @file rudp-light_vars.h
 *
 * @brief Reliable User Datagram Protocol (RUDP) - minimal implemenation (variables).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: rudp-light_vars.h 3368 2010-02-28 16:10:09Z gwitten $
 */

#ifndef RUDP_LIGHT_VARS_H
#define RUDP_LIGHT_VARS_H

#include "../types.h"
#include "rudp-light_types.h"



/**
 * @brief Debug level for this component.
 */
#define RUDP_DEBUG_LEVEL	C_MSG_WARN



/**
 * @brief Number of RUDP transmission retries.
 */
#ifdef ARCH_NS2
	#define C_RUDP_PACKET_RETRIES		10
#else
	#define C_RUDP_PACKET_RETRIES		32
#endif	// ARCH_NS2

/**
 * @brief Minimal RUDP backoff time.
 */
#ifdef ARCH_NS2
	#define C_RUDP_MIN_BACKOFF_TIME		C_NETWORK_TRAVERSAL_TIME
#else
	#define C_RUDP_MIN_BACKOFF_TIME		(0.2 * C_NETWORK_TRAVERSAL_TIME)
#endif	// ARCH_NS2

/**
 * @brief Maximal RUDP backoff time.
 */
#ifdef ARCH_NS2
	#define C_RUDP_MAX_BACKOFF_TIME		(4 * C_NETWORK_TRAVERSAL_TIME)
#else
	#define C_RUDP_MAX_BACKOFF_TIME		(2 * C_NETWORK_TRAVERSAL_TIME)
#endif	// ARCH_NS2

/**
 * @brief List of packets queued for transmission.
 */
list_t* C_RUDP_transmission_queue;

/**
 * @brief Size of cache for recently received packets.
 */
#define C_RUDP_RECEIVED_PACKET_CACHE_SIZE	256

/**
 * @brief Cache for recently received packets.
 */
packet_fingerprint_t C_RUDP_received_packet_cache[C_RUDP_RECEIVED_PACKET_CACHE_SIZE];

/**
 * @brief Statistics counter of totally transmitted packets.
 */
uint32_t C_RUDP_total_transmissions;

/**
 * @brief Statistics counter of successfully transmitted packets.
 */
uint32_t C_RUDP_successful_transmissions;

/**
 * @brief Statistics counter of not transmitted packets.
 */
uint32_t C_RUDP_failed_transmissions;

#endif	// RUDP_LIGHT_VARS_H
