/**
 * @file service_stats.h
 *
 * @brief Service statistics functions (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: service_stats.h 3421 2010-03-18 13:05:57Z gwitten $
 */

#ifndef SERVICE_STATS_H
#define SERVICE_STATS_H

#include "../types.h"
#include "service_types.h"

bool matches_host_stat_address(const void* item, const void* cmp_item);
void alloc_host_stat(SERVICE_host_stat_t** host_stat, C_addr_t address);
void init_host_stat(SERVICE_host_stat_t* host_stat, C_addr_t address);
void free_host_stat(SERVICE_host_stat_t** host_stat);
void free_host_stat_list(void* p);
float host_stat_to_cost(SERVICE_host_stat_t* host_stat);

bool matches_node_stat_address(const void* item, const void* cmp_item);
void init_node_stat(SERVICE_node_stat_t* node_stat, bool reset_time_independant_fields);
void alloc_node_stat(SERVICE_node_stat_t** node_stat, C_addr_t address);
void free_node_stat(SERVICE_node_stat_t** node_stat);
void free_node_stat_list(void* p);
float node_stat_to_cost(SERVICE_node_stat_t* node_stat);

void add_node_stat_to_host_stat(SERVICE_host_stat_t* host_stat, SERVICE_node_stat_t* node_stat);
void subtract_node_stat_to_host_stat(SERVICE_host_stat_t* host_stat, SERVICE_node_stat_t* node_stat);
void reset_service_stats(SERVICE_id_t* service_id);

void stat_service_request(SERVICE_service_t* service, size_t request_size, const net_header_t* header);

void handle_SERVICE_STATS(const net_header_t* header, const SERVICE_STATS_packet_t* stats, char* payload, size_t payload_size);
void send_statistics(SERVICE_id_t* service_id, C_addr_t destination_address);

#endif	// SERVICE_STATS_H
