/**
 * @file slp-light.h
 *
 * @brief Service Location Protocol (SLP) - minimal implemenation (header).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: slp-light.h 3275 2010-02-08 22:42:58Z gwitten $
 */

#ifndef SLP_LIGHT_H
#define SLP_LIGHT_H

#include "../types.h"
#include "slp-light_types.h"
#include "slp-light_vars.h"

// Interface to main module.
void SLP_init();
void SLP_start();
void SLP_stop();
void SLP_finalize();
void SLP_recv(const net_header_t* header, const void* payload);
bool SLP_command(int argc, const char*const* argv);

// Interface provided by this module.
void SLP_locate_service(const SERVICE_id_t* service_id, callback_function_t callback, SLP_query_result_t* query_result);
void SLP_announce_service(const SERVICE_id_t* service_id, C_addr_t replace_address);
void SLP_purge_service(const SERVICE_id_t* service_id, C_addr_t redirect_address);
void SLP_cache_update(const SERVICE_id_t* service_id, C_addr_t server);
void SLP_cache_update_with_metrics(const SERVICE_id_t* service_id, C_addr_t server, uint8_t hop_count, C_time_t delay);
void SLP_cache_remove(const SERVICE_id_t* service_id, C_addr_t server);

#endif	// SLP_LIGHT_H
