/**
 * @file slp-light_types.h
 *
 * @brief Service Location Protocol (SLP) - minimal implemenation (types).
 *
 * Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
 * Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
 * Please refer to the COPYING file for detailed licensing information.
 *
 * @author Georg Wittenburg
 *
 * $Id: slp-light_types.h 3507 2010-05-13 08:54:29Z gwitten $
 */

#ifndef SLP_LIGHT_TYPES_H
#define SLP_LIGHT_TYPES_H

#include "../types.h"
#include "../service/service_types.h"



/*
 * SLP query types.
 */

/**
 * @brief A service instance found by a SLP query.
 */
typedef struct {
	SERVICE_id_t		service_id[SERVICE_ID_LENGTH];	///< Service ID.
	C_addr_t		server;				///< Server address.
	uint8_t			hop_count;			///< Distance to the server in hops.
	C_time_t		delay;				///< Time it took the reply to arrive.
	C_time_t		last_confirmed;			///< Time at which the service was last confirmed to be at this server.
	callback_id_t		delayed_removal_callback_id;	///< ID of the callback for delayed removal.
} SLP_service_instance_t;

/**
 * @brief Result of a SLP query.
 */
typedef struct {
	C_addr_t		server;				///< Server address.
	C_errno_t		error_code;			///< Error code.
	callback_parameter_t	parameter;			///< Parameter for the callback function.
} SLP_query_result_t;

/**
 * @brief Active query data structure.
 */
typedef struct {
	SERVICE_id_t		service_id[SERVICE_ID_LENGTH];	///< Service ID.
	uint8_t			current_try;			///< Current try at finding the service.
	C_time_t		start_time;			///< Time since the current retry started.
	callback_id_t		timeout_callback_id;		///< ID of the timeout callback.
	callback_function_t	function;			///< Callback function.
	SLP_query_result_t*	result;				///< Parameter for callback function.
} SLP_query_t;



/*
 * Packet types.
 */

/**
 * @brief SLP QUERY packet.
 */
typedef struct {
	SERVICE_id_t		service_id[SERVICE_ID_LENGTH];	///< Service ID.
	uint8_t			current_try;			///< Current try at finding the service.
} SLP_QUERY_packet_t;

/**
 * @brief SLP REPLY packet.
 */
typedef struct {
	SERVICE_id_t		service_id[SERVICE_ID_LENGTH];	///< Service ID.
} SLP_REPLY_packet_t;

/**
 * @brief SLP ANNOUNCE packet.
 */
typedef struct {
	SERVICE_id_t		service_id[SERVICE_ID_LENGTH];	///< Service ID.
	C_addr_t		replace_address;		///< Address of node that the server replaces.
} SLP_ANNOUNCE_packet_t;

/**
 * @brief SLP PURGE packet.
 */
typedef struct {
	SERVICE_id_t		service_id[SERVICE_ID_LENGTH];	///< Service ID.
	C_addr_t		redirect_address;		///< Address of node to redirect requests to.
} SLP_PURGE_packet_t;

#endif	// SLP_LIGHT_TYPES_H
