#!/usr/bin/perl

##
#
# This script gathers results from a run of the service placement binary from
# several nodes and writes them to a new file. Each line is prefixed with the
# address of the host that corresponds to the particular results.
#
# Example:	eval_collect_results.pl . placement_evaluation \
#		nodes,request_frequency,run 2
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# $Id: eval_collect_results.pl 3588 2010-06-14 10:35:05Z gwitten $
#
##

use strict;

my $MAXIMAL_NETWORK_DIAMETER = 20;

my $result_section;
my $print_header = 1;
my $header;

sub process_result_lines {
	my(@result_lines) = @_;

	my $current_time_AGGREGATED = 0.0;
	my $nodes_AGGREGATED = 'n/a';
	my $area_x_AGGREGATED = 'n/a';
	my $area_y_AGGREGATED = 'n/a';
	my $setup_duration_AGGREGATED = 'n/a';
	my $run_duration_AGGREGATED = 'n/a';
	my $total_duration_AGGREGATED = 0.0;
	my $average_node_degree_AGGREGATED = 0.0;
	my $average_node_distance_AGGREGATED = 0.0;
	my $average_unicast_hop_count_AGGREGATED = 0.0;
	my $maximal_unicast_hop_count_AGGREGATED = 0;
	my @unicast_packets_per_hop_count_AGGREGATED;
	my $transmitted_unicast_packets_AGGREGATED = 0;
	my $unicast_transmission_failures_AGGREGATED = 0;
	my $received_unicast_packets_AGGREGATED = 0;
	my $unicast_pdr_AGGREGATED = 0.0;
	my $transmitted_broadcast_packets_AGGREGATED = 0;
	my $received_broadcast_packets_AGGREGATED = 0;
	my $broadcast_pdr_AGGREGATED = 0.0;
	my $transmitted_bytes_AGGREGATED = 0;
	my $received_bytes_AGGREGATED = 0;
	my $current_transmitted_datarate_AGGREGATED = 0;
	my $current_received_datarate_AGGREGATED = 0;
	my $average_transmitted_datarate_AGGREGATED = 0.0;
	my $average_received_datarate_AGGREGATED = 0.0;
	my $overhead_wrt_service_traffic_absolute_AGGREGATED = 0;
	my $overhead_wrt_service_traffic_relative_AGGREGATED = 0.0;
	my $network_mapping_overhead_AGGREGATED = 0;
	my $MAC_transmitted_packets_AGGREGATED = 'n/a';
	my $MAC_transmission_failures_AGGREGATED = 'n/a';
	my $MAC_received_packets_AGGREGATED = 'n/a';
	my $MAC_dropped_packets_capture_AGGREGATED = 'n/a';
	my $MAC_dropped_packets_collision_AGGREGATED = 'n/a';
	my $MAC_dropped_packets_transmit_AGGREGATED = 'n/a';
	my $MAC_dropped_packets_bit_error_AGGREGATED = 'n/a';
	my $MAC_transmitted_bytes_AGGREGATED = 'n/a';
	my $MAC_received_bytes_AGGREGATED = 'n/a';
	my $MAC_average_transmitted_datarate_AGGREGATED = 'n/a';
	my $MAC_average_received_datarate_AGGREGATED = 'n/a';
	my $MAC_overhead_wrt_service_traffic_absolute_AGGREGATED = 'n/a';
	my $MAC_overhead_wrt_service_traffic_relative_AGGREGATED = 'n/a';
	my $DYMO_transmitted_RREQs_AGGREGATED = 0;
	my $DYMO_transmitted_RREPs_AGGREGATED = 0;
	my $DYMO_transmitted_RERRs_AGGREGATED = 0;
	my $DYMO_transmitted_unicast_packets_AGGREGATED = 0;
	my $DYMO_received_unicast_packets_AGGREGATED = 0;
	my $DYMO_unicast_pdr_AGGREGATED = 0.0;
	my $DYMO_transmitted_broadcast_packets_AGGREGATED = 0;
	my $DYMO_received_broadcast_packets_AGGREGATED = 0;
	my $DYMO_broadcast_pdr_AGGREGATED = 0.0;
	my $RUDP_total_transmissions_AGGREGATED = 0;
	my $RUDP_successful_transmissions_AGGREGATED = 0;
	my $RUDP_pdr_AGGREGATED = 0.0;
	my $ICMP_transmitted_nops_AGGREGATED = 0;
	my $ICMP_received_nops_AGGREGATED = 0;
	my $ICMP_nop_pdr_AGGREGATED = 0.0;
	my $SLP_total_queries_AGGREGATED = 0;
	my $SLP_successful_queries_AGGREGATED = 0;
	my $SLP_recall_AGGREGATED = 0.0;
	my $SERVICE_total_data_requests_AGGREGATED = 0;
	my $SERVICE_successful_data_requests_AGGREGATED = 0;
	my $SERVICE_data_request_recall_AGGREGATED = 0.0;
	my $SERVICE_average_successful_data_request_delay_AGGREGATED = 0.0;
	my $SERVICE_total_data_updates_AGGREGATED = 0;
	my $SERVICE_successful_data_updates_AGGREGATED = 0;
	my $SERVICE_data_update_recall_AGGREGATED = 0.0;
	my $SERVICE_total_stream_requests_AGGREGATED = 0;
	my $SERVICE_successful_stream_requests_AGGREGATED = 0;
	my $SERVICE_stream_request_recall_AGGREGATED = 0.0;
	my $SERVICE_expected_stream_packets_AGGREGATED = 0;
	my $SERVICE_received_stream_packets_AGGREGATED = 0;
	my $SERVICE_stream_packet_recall_AGGREGATED = 0.0;
	my $SERVICE_total_requests_AGGREGATED = 0;
	my $SERVICE_successful_requests_AGGREGATED = 0;
	my $SERVICE_request_recall_AGGREGATED = 0.0;
	my $SERVICE_transmitted_bytes_AGGREGATED = 0;
	my $SERVICE_received_bytes_AGGREGATED = 0;
	my $SERVICE_current_transmitted_datarate_AGGREGATED = 0;
	my $SERVICE_current_received_datarate_AGGREGATED = 0;
	my $SERVICE_average_transmitted_datarate_AGGREGATED = 0.0;
	my $SERVICE_average_received_datarate_AGGREGATED = 0.0;
	my $SERVICE_average_request_hop_count_AGGREGATED = 0.0;
	my $SERVICE_maximal_request_hop_count_AGGREGATED = 0;
	my @SERVICE_request_packets_per_hop_count_AGGREGATED;
	my $SERVICE_currently_hosted_services_AGGREGATED = 0;
	my $SERVICE_average_hosted_services_AGGREGATED = 0.0;
	my $SERVICE_available_until_time_AGGREGATED = 0.0;
	my $SERVICE_sync_overhead_bytes_AGGREGATED = 0;
	my $SERVICE_sync_overhead_relative_AGGREGATED = 0.0;
	my $SERVICE_total_replications_AGGREGATED = 0;
	my $SERVICE_successful_replications_AGGREGATED = 0;
	my $SERVICE_replication_success_rate_AGGREGATED = 0.0;
	my $SERVICE_total_migrations_AGGREGATED = 0;
	my $SERVICE_successful_migrations_AGGREGATED = 0;
	my $SERVICE_migration_success_rate_AGGREGATED = 0.0;
	my $PLACEMENT_current_service_cost_AGGREGATED = 0.0;
	my $PLACEMENT_currently_optimal_service_cost_AGGREGATED = 0.0;
	my $PLACEMENT_currently_optimal_number_of_hosts_AGGREGATED = 0;
	my $PLACEMENT_currently_estimated_adaptation_cost_AGGREGATED = 0.0;
	my $PLACEMENT_total_adaptations_AGGREGATED = 0;
	my $PLACEMENT_successful_adaptations_AGGREGATED = 0;
	my $PLACEMENT_adaptation_success_rate_AGGREGATED = 0.0;




	for my $result_line (@result_lines) {

		my ($current_time, $nodes, $area_x, $area_y, $setup_duration, $run_duration, $total_duration, $average_node_degree, $average_node_distance, $average_unicast_hop_count, $maximal_unicast_hop_count, $unicast_packets_per_hop_count_0, $unicast_packets_per_hop_count_1, $unicast_packets_per_hop_count_2, $unicast_packets_per_hop_count_3, $unicast_packets_per_hop_count_4, $unicast_packets_per_hop_count_5, $unicast_packets_per_hop_count_6, $unicast_packets_per_hop_count_7, $unicast_packets_per_hop_count_8, $unicast_packets_per_hop_count_9, $unicast_packets_per_hop_count_10, $unicast_packets_per_hop_count_11, $unicast_packets_per_hop_count_12, $unicast_packets_per_hop_count_13, $unicast_packets_per_hop_count_14, $unicast_packets_per_hop_count_15, $unicast_packets_per_hop_count_16, $unicast_packets_per_hop_count_17, $unicast_packets_per_hop_count_18, $unicast_packets_per_hop_count_19, $unicast_packets_per_hop_count_20, $transmitted_unicast_packets, $unicast_transmission_failures, $received_unicast_packets, $unicast_pdr, $transmitted_broadcast_packets, $received_broadcast_packets, $broadcast_pdr, $transmitted_bytes, $received_bytes, $current_transmitted_datarate, $current_received_datarate, $average_transmitted_datarate, $average_received_datarate, $overhead_wrt_service_traffic_absolute, $overhead_wrt_service_traffic_relative, $network_mapping_overhead, $MAC_transmitted_packets, $MAC_transmission_failures, $MAC_received_packets, $MAC_dropped_packets_capture, $MAC_dropped_packets_collision, $MAC_dropped_packets_transmit, $MAC_dropped_packets_bit_error, $MAC_transmitted_bytes, $MAC_received_bytes, $MAC_average_transmitted_datarate, $MAC_average_received_datarate, $MAC_overhead_wrt_service_traffic_absolute, $MAC_overhead_wrt_service_traffic_relative, $DYMO_transmitted_RREQs, $DYMO_transmitted_RREPs, $DYMO_transmitted_RERRs, $DYMO_transmitted_unicast_packets, $DYMO_received_unicast_packets, $DYMO_unicast_pdr, $DYMO_transmitted_broadcast_packets, $DYMO_received_broadcast_packets, $DYMO_broadcast_pdr, $RUDP_total_transmissions, $RUDP_successful_transmissions, $RUDP_pdr, $ICMP_transmitted_nops, $ICMP_received_nops, $ICMP_nop_pdr, $SLP_total_queries, $SLP_successful_queries, $SLP_recall, $SERVICE_total_data_requests, $SERVICE_successful_data_requests, $SERVICE_data_request_recall, $SERVICE_average_successful_data_request_delay, $SERVICE_total_data_updates, $SERVICE_successful_data_updates, $SERVICE_data_update_recall, $SERVICE_total_stream_requests, $SERVICE_successful_stream_requests, $SERVICE_stream_request_recall, $SERVICE_expected_stream_packets, $SERVICE_received_stream_packets, $SERVICE_stream_packet_recall, $SERVICE_total_requests, $SERVICE_successful_requests, $SERVICE_request_recall, $SERVICE_transmitted_bytes, $SERVICE_received_bytes, $SERVICE_current_transmitted_datarate, $SERVICE_current_received_datarate, $SERVICE_average_transmitted_datarate, $SERVICE_average_received_datarate, $SERVICE_average_request_hop_count, $SERVICE_maximal_request_hop_count, $SERVICE_request_packets_per_hop_count_0, $SERVICE_request_packets_per_hop_count_1, $SERVICE_request_packets_per_hop_count_2, $SERVICE_request_packets_per_hop_count_3, $SERVICE_request_packets_per_hop_count_4, $SERVICE_request_packets_per_hop_count_5, $SERVICE_request_packets_per_hop_count_6, $SERVICE_request_packets_per_hop_count_7, $SERVICE_request_packets_per_hop_count_8, $SERVICE_request_packets_per_hop_count_9, $SERVICE_request_packets_per_hop_count_10, $SERVICE_request_packets_per_hop_count_11, $SERVICE_request_packets_per_hop_count_12, $SERVICE_request_packets_per_hop_count_13, $SERVICE_request_packets_per_hop_count_14, $SERVICE_request_packets_per_hop_count_15, $SERVICE_request_packets_per_hop_count_16, $SERVICE_request_packets_per_hop_count_17, $SERVICE_request_packets_per_hop_count_18, $SERVICE_request_packets_per_hop_count_19, $SERVICE_request_packets_per_hop_count_20, $SERVICE_currently_hosted_services, $SERVICE_average_hosted_services, $SERVICE_available_until_time, $SERVICE_sync_overhead_bytes, $SERVICE_sync_overhead_relative, $SERVICE_total_replications, $SERVICE_successful_replications, $SERVICE_replication_success_rate, $SERVICE_total_migrations, $SERVICE_successful_migrations, $SERVICE_migration_success_rate, $PLACEMENT_current_service_cost, $PLACEMENT_currently_optimal_service_cost, $PLACEMENT_currently_optimal_number_of_hosts, $PLACEMENT_currently_estimated_adaptation_cost, $PLACEMENT_total_adaptations, $PLACEMENT_successful_adaptations, $PLACEMENT_adaptation_success_rate) = split(/,/, $result_line);

		my @unicast_packets_per_hop_count = ($unicast_packets_per_hop_count_0, $unicast_packets_per_hop_count_1, $unicast_packets_per_hop_count_2, $unicast_packets_per_hop_count_3, $unicast_packets_per_hop_count_4, $unicast_packets_per_hop_count_5, $unicast_packets_per_hop_count_6, $unicast_packets_per_hop_count_7, $unicast_packets_per_hop_count_8, $unicast_packets_per_hop_count_9, $unicast_packets_per_hop_count_10, $unicast_packets_per_hop_count_11, $unicast_packets_per_hop_count_12, $unicast_packets_per_hop_count_13, $unicast_packets_per_hop_count_14, $unicast_packets_per_hop_count_15, $unicast_packets_per_hop_count_16, $unicast_packets_per_hop_count_17, $unicast_packets_per_hop_count_18, $unicast_packets_per_hop_count_19, $unicast_packets_per_hop_count_20);
		my @SERVICE_request_packets_per_hop_count = ($SERVICE_request_packets_per_hop_count_0, $SERVICE_request_packets_per_hop_count_1, $SERVICE_request_packets_per_hop_count_2, $SERVICE_request_packets_per_hop_count_3, $SERVICE_request_packets_per_hop_count_4, $SERVICE_request_packets_per_hop_count_5, $SERVICE_request_packets_per_hop_count_6, $SERVICE_request_packets_per_hop_count_7, $SERVICE_request_packets_per_hop_count_8, $SERVICE_request_packets_per_hop_count_9, $SERVICE_request_packets_per_hop_count_10, $SERVICE_request_packets_per_hop_count_11, $SERVICE_request_packets_per_hop_count_12, $SERVICE_request_packets_per_hop_count_13, $SERVICE_request_packets_per_hop_count_14, $SERVICE_request_packets_per_hop_count_15, $SERVICE_request_packets_per_hop_count_16, $SERVICE_request_packets_per_hop_count_17, $SERVICE_request_packets_per_hop_count_18, $SERVICE_request_packets_per_hop_count_19, $SERVICE_request_packets_per_hop_count_20);

		$current_time_AGGREGATED += $current_time;
		# $nodes_AGGREGATED
		# $area_x_AGGREGATED
		# $area_y_AGGREGATED
		# $setup_duration_AGGREGATED
		# $run_duration_AGGREGATED
		$total_duration_AGGREGATED += $total_duration;
		$average_node_degree_AGGREGATED += $average_node_degree;
		$average_node_distance_AGGREGATED += $average_node_distance;
		# $average_unicast_hop_count_AGGREGATED
		if($maximal_unicast_hop_count_AGGREGATED < $maximal_unicast_hop_count) {
			$maximal_unicast_hop_count_AGGREGATED = $maximal_unicast_hop_count;
		}
		for my $i (0..$MAXIMAL_NETWORK_DIAMETER) {
			$unicast_packets_per_hop_count_AGGREGATED[$i] += $unicast_packets_per_hop_count[$i];
		}
		$transmitted_unicast_packets_AGGREGATED += $transmitted_unicast_packets;
		$unicast_transmission_failures_AGGREGATED += $unicast_transmission_failures;
		$received_unicast_packets_AGGREGATED += $received_unicast_packets;
		# $unicast_pdr_AGGREGATED
		$transmitted_broadcast_packets_AGGREGATED += $transmitted_broadcast_packets;
		$received_broadcast_packets_AGGREGATED += $received_broadcast_packets;
		# $broadcast_pdr_AGGREGATED
		$transmitted_bytes_AGGREGATED += $transmitted_bytes;
		$received_bytes_AGGREGATED += $received_bytes;
		$current_transmitted_datarate_AGGREGATED += $current_transmitted_datarate;
		$current_received_datarate_AGGREGATED += $current_received_datarate;
		# $average_transmitted_datarate_AGGREGATED
		# $average_received_datarate_AGGREGATED
		$overhead_wrt_service_traffic_absolute_AGGREGATED += $overhead_wrt_service_traffic_absolute;
		$overhead_wrt_service_traffic_relative_AGGREGATED += $overhead_wrt_service_traffic_relative;
		$network_mapping_overhead_AGGREGATED += $network_mapping_overhead;
		$DYMO_transmitted_RREQs_AGGREGATED += $DYMO_transmitted_RREQs;
		$DYMO_transmitted_RREPs_AGGREGATED += $DYMO_transmitted_RREPs;
		$DYMO_transmitted_RERRs_AGGREGATED += $DYMO_transmitted_RERRs;
		$DYMO_transmitted_unicast_packets_AGGREGATED += $DYMO_transmitted_unicast_packets;
		$DYMO_received_unicast_packets_AGGREGATED += $DYMO_received_unicast_packets;
		# $DYMO_unicast_pdr_AGGREGATED
		$DYMO_transmitted_broadcast_packets_AGGREGATED += $DYMO_transmitted_broadcast_packets;
		$DYMO_received_broadcast_packets_AGGREGATED += $DYMO_received_broadcast_packets;
		# $DYMO_broadcast_pdr_AGGREGATED
		$RUDP_total_transmissions_AGGREGATED += $RUDP_total_transmissions;
		$RUDP_successful_transmissions_AGGREGATED += $RUDP_successful_transmissions;
		# $RUDP_pdr_AGGREGATED
		$ICMP_transmitted_nops_AGGREGATED += $ICMP_transmitted_nops;
		$ICMP_received_nops_AGGREGATED += $ICMP_received_nops;
		# $ICMP_nop_pdr_AGGREGATED
		$SLP_total_queries_AGGREGATED += $SLP_total_queries;
		$SLP_successful_queries_AGGREGATED += $SLP_successful_queries;
		# $SLP_recall_AGGREGATED
		$SERVICE_total_data_requests_AGGREGATED += $SERVICE_total_data_requests;
		$SERVICE_successful_data_requests_AGGREGATED += $SERVICE_successful_data_requests;
		# $SERVICE_data_request_recall_AGGREGATED
		$SERVICE_average_successful_data_request_delay_AGGREGATED += $SERVICE_average_successful_data_request_delay;
		$SERVICE_total_data_updates_AGGREGATED += $SERVICE_total_data_updates;
		$SERVICE_successful_data_updates_AGGREGATED += $SERVICE_successful_data_updates;
		# $SERVICE_data_update_recall_AGGREGATED
		$SERVICE_total_stream_requests_AGGREGATED += $SERVICE_total_stream_requests;
		$SERVICE_successful_stream_requests_AGGREGATED += $SERVICE_successful_stream_requests;
		# $SERVICE_stream_request_recall_AGGREGATED
		$SERVICE_expected_stream_packets_AGGREGATED += $SERVICE_expected_stream_packets;
		$SERVICE_received_stream_packets_AGGREGATED += $SERVICE_received_stream_packets;
		# $SERVICE_stream_packet_recall_AGGREGATED
		$SERVICE_total_requests_AGGREGATED += $SERVICE_total_requests;
		$SERVICE_successful_requests_AGGREGATED += $SERVICE_successful_requests;
		# $SERVICE_request_recall_AGGREGATED
		$SERVICE_transmitted_bytes_AGGREGATED += $SERVICE_transmitted_bytes;
		$SERVICE_received_bytes_AGGREGATED += $SERVICE_received_bytes;
		$SERVICE_current_transmitted_datarate_AGGREGATED += $SERVICE_current_transmitted_datarate;
		$SERVICE_current_received_datarate_AGGREGATED += $SERVICE_current_received_datarate;
		# $SERVICE_average_transmitted_datarate_AGGREGATED
		# $SERVICE_average_received_datarate_AGGREGATED
		# $SERVICE_average_request_hop_count_AGGREGATED
		if($SERVICE_maximal_request_hop_count_AGGREGATED < $SERVICE_maximal_request_hop_count) {
			$SERVICE_maximal_request_hop_count_AGGREGATED = $SERVICE_maximal_request_hop_count;
		}
		for my $i (0..$MAXIMAL_NETWORK_DIAMETER) {
			$SERVICE_request_packets_per_hop_count_AGGREGATED[$i] += $SERVICE_request_packets_per_hop_count[$i];
		}
		$SERVICE_currently_hosted_services_AGGREGATED += $SERVICE_currently_hosted_services;
		$SERVICE_average_hosted_services_AGGREGATED += $SERVICE_average_hosted_services;
		if($SERVICE_available_until_time_AGGREGATED < $SERVICE_available_until_time) {
			$SERVICE_available_until_time_AGGREGATED = $SERVICE_available_until_time;
		}
		$SERVICE_sync_overhead_bytes_AGGREGATED += $SERVICE_sync_overhead_bytes;
		# $SERVICE_sync_overhead_relative_AGGREGATED
		$SERVICE_total_replications_AGGREGATED += $SERVICE_total_replications;
		$SERVICE_successful_replications_AGGREGATED += $SERVICE_successful_replications;
		# $SERVICE_replication_success_rate_AGGREGATED
		$SERVICE_total_migrations_AGGREGATED += $SERVICE_total_migrations;
		$SERVICE_successful_migrations_AGGREGATED += $SERVICE_successful_migrations;
		# $SERVICE_migration_success_rate_AGGREGATED
		$PLACEMENT_current_service_cost_AGGREGATED += $PLACEMENT_current_service_cost;
		$PLACEMENT_currently_optimal_service_cost_AGGREGATED += $PLACEMENT_currently_optimal_service_cost;
		$PLACEMENT_currently_optimal_number_of_hosts_AGGREGATED += $PLACEMENT_currently_optimal_number_of_hosts;
		$PLACEMENT_currently_estimated_adaptation_cost_AGGREGATED += $PLACEMENT_currently_estimated_adaptation_cost;
		$PLACEMENT_total_adaptations_AGGREGATED += $PLACEMENT_total_adaptations;
		$PLACEMENT_successful_adaptations_AGGREGATED += $PLACEMENT_successful_adaptations;
		# $PLACEMENT_adaptation_success_rate_AGGREGATED

		if(
			($SERVICE_total_requests > 0)
			&& ($SERVICE_successful_requests == 0)
			&& ($SLP_total_queries > 0)
			&& ($SLP_successful_queries == 0)
		) {
			print "WARNING: $SERVICE_total_requests service request(s), but none of them was successful ($SERVICE_successful_requests/$SLP_total_queries SLP recall).\n";
# 			exit 1;
		}
	}

	$current_time_AGGREGATED /= scalar(@result_lines);
	# $nodes_AGGREGATED
	# $area_x_AGGREGATED
	# $area_y_AGGREGATED
	# $setup_duration_AGGREGATED
	# $run_duration_AGGREGATED
	$total_duration_AGGREGATED /= scalar(@result_lines);

	# Remove 10s from the total duration that were added in eval_generate_scenario.pl to work around initialization issues.
	$total_duration_AGGREGATED -= 10.0;

	$average_node_degree_AGGREGATED /= scalar(@result_lines);
	$average_node_distance_AGGREGATED /= scalar(@result_lines);

	my $total_unicast_packets_with_hop_count = 0;
	for my $i (0..$MAXIMAL_NETWORK_DIAMETER) {
		$average_unicast_hop_count_AGGREGATED += $unicast_packets_per_hop_count_AGGREGATED[$i] * $i;
		$total_unicast_packets_with_hop_count += $unicast_packets_per_hop_count_AGGREGATED[$i];
	}
	if($total_unicast_packets_with_hop_count > 0) {
		$average_unicast_hop_count_AGGREGATED /= $total_unicast_packets_with_hop_count;
	}

	# $maximal_unicast_hop_count_AGGREGATED
	# @unicast_packets_per_hop_count_AGGREGATED
	# $transmitted_unicast_packets_AGGREGATED
	# $unicast_transmission_failures_AGGREGATED
	# $received_unicast_packets_AGGREGATED
	if($transmitted_unicast_packets_AGGREGATED > 0) {
		$unicast_pdr_AGGREGATED = ($received_unicast_packets_AGGREGATED / $transmitted_unicast_packets_AGGREGATED);
	}
	# $transmitted_broadcast_packets_AGGREGATED
	# $received_broadcast_packets_AGGREGATED
	if($transmitted_broadcast_packets_AGGREGATED > 0) {
		$broadcast_pdr_AGGREGATED = ($received_broadcast_packets_AGGREGATED / $transmitted_broadcast_packets_AGGREGATED);
	}
	# $transmitted_bytes_AGGREGATED
	# $received_bytes_AGGREGATED
	$current_transmitted_datarate_AGGREGATED /= scalar(@result_lines);
	$current_received_datarate_AGGREGATED /= scalar(@result_lines);
	if($total_duration_AGGREGATED > 0) {
		$average_transmitted_datarate_AGGREGATED = ($transmitted_bytes_AGGREGATED / $total_duration_AGGREGATED);
	}
	if($total_duration_AGGREGATED > 0) {
		$average_received_datarate_AGGREGATED = ($received_bytes_AGGREGATED / $total_duration_AGGREGATED);
	}
	# $overhead_wrt_service_traffic_absolute_AGGREGATED
	$overhead_wrt_service_traffic_relative_AGGREGATED /= scalar(@result_lines);
	# $network_mapping_overhead_AGGREGATED
	# MAC_transmitted_packets_AGGREGATED
	# MAC_transmission_failures_AGGREGATED
	# MAC_received_packets_AGGREGATED
	# MAC_dropped_packets_capture_AGGREGATED
	# MAC_dropped_packets_collision_AGGREGATED
	# MAC_dropped_packets_transmit_AGGREGATED
	# MAC_dropped_packets_bit_error_AGGREGATED
	# MAC_transmitted_bytes_AGGREGATED
	# MAC_received_bytes_AGGREGATED
	# MAC_average_transmitted_datarate_AGGREGATED
	# MAC_average_received_datarate_AGGREGATED
	# MAC_overhead_wrt_service_traffic_absolute_AGGREGATED
	# MAC_overhead_wrt_service_traffic_relative_AGGREGATED
	# $DYMO_transmitted_RREQs_AGGREGATED
	# $DYMO_transmitted_RREPs_AGGREGATED
	# $DYMO_transmitted_RERRs_AGGREGATED
	# $DYMO_transmitted_unicast_packets_AGGREGATED
	# $DYMO_received_unicast_packets_AGGREGATED
	if($DYMO_transmitted_unicast_packets_AGGREGATED > 0) {
		$DYMO_unicast_pdr_AGGREGATED = ($DYMO_received_unicast_packets_AGGREGATED / $DYMO_transmitted_unicast_packets_AGGREGATED);
	}
	# $DYMO_transmitted_broadcast_packets_AGGREGATED
	# $DYMO_received_broadcast_packets_AGGREGATED
	if($DYMO_transmitted_broadcast_packets_AGGREGATED > 0) {
		$DYMO_broadcast_pdr_AGGREGATED = ($DYMO_received_broadcast_packets_AGGREGATED / $DYMO_transmitted_broadcast_packets_AGGREGATED);
	}
	# $RUDP_total_transmissions_AGGREGATED
	# $RUDP_successful_transmissions_AGGREGATED
	if($RUDP_total_transmissions_AGGREGATED > 0) {
		$RUDP_pdr_AGGREGATED = ($RUDP_successful_transmissions_AGGREGATED / $RUDP_total_transmissions_AGGREGATED);
	}
	# $ICMP_transmitted_nops_AGGREGATED
	# $ICMP_received_nops_AGGREGATED
	if($ICMP_transmitted_nops_AGGREGATED > 0) {
		$ICMP_nop_pdr_AGGREGATED = ($ICMP_received_nops_AGGREGATED / $ICMP_transmitted_nops_AGGREGATED);
	}
	# $SLP_total_queries_AGGREGATED
	# $SLP_successful_queries_AGGREGATED
	if($SLP_total_queries_AGGREGATED > 0) {
		$SLP_recall_AGGREGATED = ($SLP_successful_queries_AGGREGATED / $SLP_total_queries_AGGREGATED);
	}
	# $SERVICE_total_data_requests_AGGREGATED
	# $SERVICE_successful_data_requests_AGGREGATED
	if($SERVICE_total_data_requests_AGGREGATED > 0) {
		$SERVICE_data_request_recall_AGGREGATED = ($SERVICE_successful_data_requests_AGGREGATED / $SERVICE_total_data_requests_AGGREGATED);
	}
	$SERVICE_average_successful_data_request_delay_AGGREGATED /= scalar(@result_lines);
	# $SERVICE_total_data_updates_AGGREGATED
	# $SERVICE_successful_data_updates_AGGREGATED
	if($SERVICE_total_data_updates_AGGREGATED > 0) {
		$SERVICE_data_update_recall_AGGREGATED = ($SERVICE_successful_data_updates_AGGREGATED / $SERVICE_total_data_updates_AGGREGATED);
	}
	# $SERVICE_total_stream_requests_AGGREGATED
	# $SERVICE_successful_stream_requests_AGGREGATED
	if($SERVICE_total_stream_requests_AGGREGATED > 0) {
		$SERVICE_stream_request_recall_AGGREGATED = ($SERVICE_successful_stream_requests_AGGREGATED / $SERVICE_total_stream_requests_AGGREGATED);
	}
	# $SERVICE_expected_stream_packets_AGGREGATED
	# $SERVICE_received_stream_packets_AGGREGATED
	if($SERVICE_expected_stream_packets_AGGREGATED > 0) {
		$SERVICE_stream_packet_recall_AGGREGATED = ($SERVICE_received_stream_packets_AGGREGATED / $SERVICE_expected_stream_packets_AGGREGATED);
	}
	# $SERVICE_total_requests_AGGREGATED
	# $SERVICE_successful_requests_AGGREGATED
	if($SERVICE_total_requests_AGGREGATED > 0) {
		$SERVICE_request_recall_AGGREGATED = ($SERVICE_successful_requests_AGGREGATED / $SERVICE_total_requests_AGGREGATED);
	}
	# $SERVICE_transmitted_bytes_AGGREGATED
	# $SERVICE_received_bytes_AGGREGATED
	$SERVICE_current_transmitted_datarate_AGGREGATED /= scalar(@result_lines);
	$SERVICE_current_received_datarate_AGGREGATED /= scalar(@result_lines);
	if($total_duration_AGGREGATED > 0) {
		$SERVICE_average_transmitted_datarate_AGGREGATED = ($SERVICE_transmitted_bytes_AGGREGATED / $total_duration_AGGREGATED);
	}
	if($total_duration_AGGREGATED > 0) {
		$SERVICE_average_received_datarate_AGGREGATED = ($SERVICE_received_bytes_AGGREGATED / $total_duration_AGGREGATED);
	}

	my $SERVICE_total_request_packets_per_hop_count = 0;
	for my $i (0..$MAXIMAL_NETWORK_DIAMETER) {
		$SERVICE_average_request_hop_count_AGGREGATED += $SERVICE_request_packets_per_hop_count_AGGREGATED[$i] * $i;
		$SERVICE_total_request_packets_per_hop_count += $SERVICE_request_packets_per_hop_count_AGGREGATED[$i];
	}
	if($SERVICE_total_request_packets_per_hop_count > 0) {
		$SERVICE_average_request_hop_count_AGGREGATED /= $SERVICE_total_request_packets_per_hop_count;
	}

	# $SERVICE_maximal_request_hop_count_AGGREGATED
	# @SERVICE_request_packets_per_hop_count_AGGREGATED
	# $SERVICE_currently_hosted_services_AGGREGATED
	# $SERVICE_average_hosted_services_AGGREGATED
	# $SERVICE_available_until_time_AGGREGATED
	# $SERVICE_sync_overhead_bytes_AGGREGATED
	if($SERVICE_transmitted_bytes_AGGREGATED > 0) {
		$SERVICE_sync_overhead_relative_AGGREGATED = ($SERVICE_sync_overhead_bytes_AGGREGATED / $SERVICE_transmitted_bytes_AGGREGATED);
	}
	# $SERVICE_total_replications_AGGREGATED
	# $SERVICE_successful_replications_AGGREGATED
	if($SERVICE_total_replications_AGGREGATED > 0) {
		$SERVICE_replication_success_rate_AGGREGATED = ($SERVICE_successful_replications_AGGREGATED / $SERVICE_total_replications_AGGREGATED);
	}
	# $SERVICE_total_migrations_AGGREGATED
	# $SERVICE_successful_migrations_AGGREGATED
	if($SERVICE_total_migrations_AGGREGATED > 0) {
		$SERVICE_migration_success_rate_AGGREGATED = ($SERVICE_successful_migrations_AGGREGATED / $SERVICE_total_migrations_AGGREGATED);
	}
	# $PLACEMENT_current_service_cost_AGGREGATED
	# $PLACEMENT_currently_optimal_service_cost_AGGREGATED
	# $PLACEMENT_currently_optimal_number_of_hosts_AGGREGATED
	# $PLACEMENT_currently_estimated_adaptation_cost_AGGREGATED
	# $PLACEMENT_total_adaptations_AGGREGATED
	# $PLACEMENT_successful_adaptations_AGGREGATED
	if($PLACEMENT_total_adaptations_AGGREGATED > 0) {
		$PLACEMENT_adaptation_success_rate_AGGREGATED = ($PLACEMENT_successful_adaptations_AGGREGATED / $PLACEMENT_total_adaptations_AGGREGATED);
	}

	return "$current_time_AGGREGATED,$nodes_AGGREGATED,$area_x_AGGREGATED,$area_y_AGGREGATED,$setup_duration_AGGREGATED,$run_duration_AGGREGATED,$total_duration_AGGREGATED,$average_node_degree_AGGREGATED,$average_node_distance_AGGREGATED,$average_unicast_hop_count_AGGREGATED,$maximal_unicast_hop_count_AGGREGATED,".join(',', @unicast_packets_per_hop_count_AGGREGATED).",$transmitted_unicast_packets_AGGREGATED,$unicast_transmission_failures_AGGREGATED,$received_unicast_packets_AGGREGATED,$unicast_pdr_AGGREGATED,$transmitted_broadcast_packets_AGGREGATED,$received_broadcast_packets_AGGREGATED,$broadcast_pdr_AGGREGATED,$transmitted_bytes_AGGREGATED,$received_bytes_AGGREGATED,$current_transmitted_datarate_AGGREGATED,$current_received_datarate_AGGREGATED,$average_transmitted_datarate_AGGREGATED,$average_received_datarate_AGGREGATED,$overhead_wrt_service_traffic_absolute_AGGREGATED,$overhead_wrt_service_traffic_relative_AGGREGATED,$network_mapping_overhead_AGGREGATED,$MAC_transmitted_packets_AGGREGATED,$MAC_transmission_failures_AGGREGATED,$MAC_received_packets_AGGREGATED,$MAC_dropped_packets_capture_AGGREGATED,$MAC_dropped_packets_collision_AGGREGATED,$MAC_dropped_packets_transmit_AGGREGATED,$MAC_dropped_packets_bit_error_AGGREGATED,$MAC_transmitted_bytes_AGGREGATED,$MAC_received_bytes_AGGREGATED,$MAC_average_transmitted_datarate_AGGREGATED,$MAC_average_received_datarate_AGGREGATED,$MAC_overhead_wrt_service_traffic_absolute_AGGREGATED,$MAC_overhead_wrt_service_traffic_relative_AGGREGATED,$DYMO_transmitted_RREQs_AGGREGATED,$DYMO_transmitted_RREPs_AGGREGATED,$DYMO_transmitted_RERRs_AGGREGATED,$DYMO_transmitted_unicast_packets_AGGREGATED,$DYMO_received_unicast_packets_AGGREGATED,$DYMO_unicast_pdr_AGGREGATED,$DYMO_transmitted_broadcast_packets_AGGREGATED,$DYMO_received_broadcast_packets_AGGREGATED,$DYMO_broadcast_pdr_AGGREGATED,$RUDP_total_transmissions_AGGREGATED,$RUDP_successful_transmissions_AGGREGATED,$RUDP_pdr_AGGREGATED,$ICMP_transmitted_nops_AGGREGATED,$ICMP_received_nops_AGGREGATED,$ICMP_nop_pdr_AGGREGATED,$SLP_total_queries_AGGREGATED,$SLP_successful_queries_AGGREGATED,$SLP_recall_AGGREGATED,$SERVICE_total_data_requests_AGGREGATED,$SERVICE_successful_data_requests_AGGREGATED,$SERVICE_data_request_recall_AGGREGATED,$SERVICE_average_successful_data_request_delay_AGGREGATED,$SERVICE_total_data_updates_AGGREGATED,$SERVICE_successful_data_updates_AGGREGATED,$SERVICE_data_update_recall_AGGREGATED,$SERVICE_total_stream_requests_AGGREGATED,$SERVICE_successful_stream_requests_AGGREGATED,$SERVICE_stream_request_recall_AGGREGATED,$SERVICE_expected_stream_packets_AGGREGATED,$SERVICE_received_stream_packets_AGGREGATED,$SERVICE_stream_packet_recall_AGGREGATED,$SERVICE_total_requests_AGGREGATED,$SERVICE_successful_requests_AGGREGATED,$SERVICE_request_recall_AGGREGATED,$SERVICE_transmitted_bytes_AGGREGATED,$SERVICE_received_bytes_AGGREGATED,$SERVICE_current_transmitted_datarate_AGGREGATED,$SERVICE_current_received_datarate_AGGREGATED,$SERVICE_average_transmitted_datarate_AGGREGATED,$SERVICE_average_received_datarate_AGGREGATED,$SERVICE_average_request_hop_count_AGGREGATED,$SERVICE_maximal_request_hop_count_AGGREGATED,".join(',', @SERVICE_request_packets_per_hop_count_AGGREGATED).",$SERVICE_currently_hosted_services_AGGREGATED,$SERVICE_average_hosted_services_AGGREGATED,$SERVICE_available_until_time_AGGREGATED,$SERVICE_sync_overhead_bytes_AGGREGATED,$SERVICE_sync_overhead_relative_AGGREGATED,$SERVICE_total_replications_AGGREGATED,$SERVICE_successful_replications_AGGREGATED,$SERVICE_replication_success_rate_AGGREGATED,$SERVICE_total_migrations_AGGREGATED,$SERVICE_successful_migrations_AGGREGATED,$SERVICE_migration_success_rate_AGGREGATED,$PLACEMENT_current_service_cost_AGGREGATED,$PLACEMENT_currently_optimal_service_cost_AGGREGATED,$PLACEMENT_currently_optimal_number_of_hosts_AGGREGATED,$PLACEMENT_currently_estimated_adaptation_cost_AGGREGATED,$PLACEMENT_total_adaptations_AGGREGATED,$PLACEMENT_successful_adaptations_AGGREGATED,$PLACEMENT_adaptation_success_rate_AGGREGATED";
}

sub process_output_files {
	my($current_directory, $output_files_ref) = @_;
	my @output_files = @{$output_files_ref};
	my @result_lines;

	for my $output_file (sort @output_files) {
		my $run_complete_flag = 0;
		my $current_result_section = 0;

		# print "Processing $output_file ...\n";
		open(my $OUTPUT_FILE, "<$current_directory/$output_file") or die $!;
		OUTPUT_LINE: while(<$OUTPUT_FILE>) {
			if($_ =~ /Run complete/) {
				$run_complete_flag = 1;
			}
			if($_ =~ /^###$/) {
	
				# Skip to the correct result section.
				$current_result_section++;
				if($current_result_section != $result_section) {
					next OUTPUT_LINE;
				}
	
				# Check whether the run has been completed.
				if(!$run_complete_flag) {
					die "No run complete flag before results section #$result_section in $current_directory/$output_file";
				}
	
				my $header_line = 1;
				# Process result section.
				DATA_LINE: while(<$OUTPUT_FILE>) {
	
					# Break on empty line.
					if(($_ =~ /^$/) || ($_ =~ /^.$/)) {
						last DATA_LINE;
					}
	
					# Print header, but only once.
					if($header_line) {
						if($print_header) {
							$header .= ','.$_;
							$print_header = 0;
						}
						$header_line = 0;
						next DATA_LINE;
					}

					chomp($_);
					push(@result_lines, $_);
	
				}
			}
		}
		close($OUTPUT_FILE);
	
		# Check whether the run has been completed.
		if(!$run_complete_flag) {
			die "No run complete flag in $current_directory/$output_file";
		}
	}
	return process_result_lines(@result_lines);
}

sub process_variables {
	my($current_directory, $variable_names_ref) = @_;
	my @variable_names = @{$variable_names_ref};
	my @output_lines;

	if(scalar(@variable_names) > 0) {
		my $current_variable_name = shift(@variable_names);
		my @next_directories;
		opendir(my $DIR, $current_directory) or die $!;
		while(my $next_directory = readdir($DIR)) {
			if(
				($next_directory =~ /$current_variable_name\_.+/)
				&& (-d "$current_directory/$next_directory")
			) {
				push(@next_directories, $next_directory);
			}
		}
		if(scalar(@next_directories) == 0) {
			die "No subdirectories matching variable \"$current_variable_name\" found in $current_directory";
		}
		for my $next_directory (sort @next_directories) {
			$next_directory =~ /$current_variable_name\_(.+)/;
			my $current_variable_value =  $1;
			# print "Processing $current_variable_name = $current_variable_value...\n";
			my @next_output_lines = process_variables("$current_directory/$next_directory", \@variable_names);
			if(scalar(@next_output_lines) > 0) {
				for my $i (0..$#next_output_lines) {
					$next_output_lines[$i] = "$current_variable_value,$next_output_lines[$i]";
				}
				push(@output_lines, @next_output_lines);
			}
		}
	} else {
		my @output_files;
		opendir(my $DIR, $current_directory) or die $!;
		while(my $output_file = readdir($DIR)) {
			if(
				($output_file =~ /node_.+_out.txt\Z/)
				&& (-r "$current_directory/$output_file")
			) {
				push(@output_files, $output_file);
			}
		}
		if(scalar(@output_files) > 0) {
			print "Processing output in $current_directory (".scalar(@output_files)." nodes)...\n";
			@output_lines = (process_output_files($current_directory, \@output_files));
		} else {
			print "No output in $current_directory ...\n";
			@output_lines = ();
		}
	}
	return @output_lines;
}

if((scalar(@ARGV) < 3) || (scalar(@ARGV) > 4)) {
	print "Usage:\teval_collect_results.pl out_directory base_name variable_name(s) [result_section]\n\n";
	exit(1);
}

my $current_directory = $ARGV[0];
my $base_name = $ARGV[1];
my @variable_names = split(/,/, $ARGV[2]);
$result_section = 1;
if(scalar(@ARGV) == 4) {
	$result_section = $ARGV[3];
}
$header = $ARGV[2];

print "Output files are in $current_directory/.\n";
print "Base name is \"$base_name\", variables names are [".join(',', @variable_names)."].\n";
print "\n";

if(-e "$base_name.csv") {
	die "Output file $base_name.csv already exists";
}

my @output_lines = process_variables($current_directory, \@variable_names);

open(my $CSV_FILE, ">$base_name.csv") or die $!;
print $CSV_FILE $header;
for my $output_line (@output_lines) {
	print $CSV_FILE "$output_line\n";
}
close($CSV_FILE);
print "Done.\n";
