#!/usr/bin/perl

use strict;

my $SRVPLACE_DIRECTORY = "/home/fenn/gwitten/usr/bin";
my $BASE_DIRECTORY = "/home/fenn/gwitten/RAID/sp_evaluation";
my $TOPOLOGY_DIRECTORY = "/home/fenn/gwitten/RAID/sp_evaluation/topologies";

my @PLACEMENTS = (

	# 100% links
	{
		id		=> "none",
		binary		=> "srvplace.sp_none_POOL_100P_LINKS",
		directory	=> "placement_evaluation_none_POOL_100P_LINKS",
	},
	{
		id		=> "spi_gcsi",
		binary		=> "srvplace.sp_spi_POOL_100P_LINKS",
		directory	=> "placement_evaluation_spi_gcsi_POOL_100P_LINKS",
	},
	{
		id		=> "spi_gcmi",
		binary		=> "srvplace.sp_spi_POOL_100P_LINKS",
		directory	=> "placement_evaluation_spi_gcmi_POOL_100P_LINKS",
	},

	# 90% links
	{
		id		=> "none",
		binary		=> "srvplace.sp_none_POOL_90P_LINKS",
		directory	=> "placement_evaluation_none_POOL_90P_LINKS",
	},
	{
		id		=> "spi_gcsi",
		binary		=> "srvplace.sp_spi_POOL_90P_LINKS",
		directory	=> "placement_evaluation_spi_gcsi_POOL_90P_LINKS",
	},
	{
		id		=> "spi_gcmi",
		binary		=> "srvplace.sp_spi_POOL_90P_LINKS",
		directory	=> "placement_evaluation_spi_gcmi_POOL_90P_LINKS",
	},

	# 80% links
	{
		id		=> "none",
		binary		=> "srvplace.sp_none_POOL_80P_LINKS",
		directory	=> "placement_evaluation_none_POOL_80P_LINKS",
	},
	{
		id		=> "spi_gcsi",
		binary		=> "srvplace.sp_spi_POOL_80P_LINKS",
		directory	=> "placement_evaluation_spi_gcsi_POOL_80P_LINKS",
	},
	{
		id		=> "spi_gcmi",
		binary		=> "srvplace.sp_spi_POOL_80P_LINKS",
		directory	=> "placement_evaluation_spi_gcmi_POOL_80P_LINKS",
	},

	# 70% links
	{
		id		=> "none",
		binary		=> "srvplace.sp_none_POOL_70P_LINKS",
		directory	=> "placement_evaluation_none_POOL_70P_LINKS",
	},
	{
		id		=> "spi_gcsi",
		binary		=> "srvplace.sp_spi_POOL_70P_LINKS",
		directory	=> "placement_evaluation_spi_gcsi_POOL_70P_LINKS",
	},
	{
		id		=> "spi_gcmi",
		binary		=> "srvplace.sp_spi_POOL_70P_LINKS",
		directory	=> "placement_evaluation_spi_gcmi_POOL_70P_LINKS",
	},
);

my @SERVICE_SIZES = (100000);
my @SYNC_RATIOS = ("0.010");
# my @NODES = (16, 25, 36, 49);
my @NODES = (36);
my @ACTIVE_NODE_RATIOS = (0.5);
# my @REQUEST_FREQUENCIES = ("0.1", "0.5", "1.0", "2.0", "5.0");
my @REQUEST_FREQUENCIES = ("1.0", "5.0", "2.0", "0.1", "0.5");
my $RUNS = 30;

if(!-e $SRVPLACE_DIRECTORY) {
	print "Service placement base directory $SRVPLACE_DIRECTORY does not exist.\n\n";
	exit(1);
}
if(!-e $BASE_DIRECTORY) {
	print "Base directory $BASE_DIRECTORY does not exist.\n\n";
	exit(1);
}
if(($TOPOLOGY_DIRECTORY ne "") && (!-e $TOPOLOGY_DIRECTORY)) {
	print "Topology directory $TOPOLOGY_DIRECTORY does not exist.\n\n";
	exit(1);
}

for my $run (1..$RUNS) {
	if($run < 10) {
		$run = '0'.$run;
	}
for my $request_frequency (@REQUEST_FREQUENCIES) {
for my $active_node_ratio (@ACTIVE_NODE_RATIOS) {
for my $nodes (@NODES) {
for my $sync_ratio (@SYNC_RATIOS) {
for my $service_size (@SERVICE_SIZES) {
for my $placement_ref (@PLACEMENTS) {
	my %placement = %{$placement_ref};

	my $dir = "$placement{directory}/service_size_$service_size/sync_ratio_$sync_ratio/nodes_$nodes/activity_factor_$active_node_ratio/request_frequency_$request_frequency/run_$run/";
	chdir($dir);

	my $start_time = time();
	print "\n[".localtime($start_time)."] Starting run for $dir...\n";
	if($TOPOLOGY_DIRECTORY ne "") {
		print `eval_run.pl $SRVPLACE_DIRECTORY/$placement{binary} $BASE_DIRECTORY/$dir $BASE_DIRECTORY/$dir $TOPOLOGY_DIRECTORY/$nodes\_nodes/`;
	} else {
		print `eval_run.pl $SRVPLACE_DIRECTORY/$placement{binary} $BASE_DIRECTORY/$dir $BASE_DIRECTORY/$dir`;
	}
	my $run_errno = $?;
	my $stop_time = time();
	my $duration = $stop_time - $start_time;
	my $hours = int($duration / 3600);
	my $minutes = int($duration / 60);
	if($minutes < 10) {
		$minutes = '0'.$minutes;
	}
	my $seconds = $duration % 60;
	if($seconds < 10) {
		$seconds = '0'.$seconds;
	}
	if($run_errno != 0) {
		print "[".localtime($stop_time)."] Run for $dir failed after $hours:$minutes:$seconds ($run_errno).\n";
		print "Aborting...\n\n";
		exit(1);
	}
	print "[".localtime($stop_time)."] Run for $dir completed after $hours:$minutes:$seconds.\n";
	chdir($BASE_DIRECTORY);

}
}
}
}
}
}
}
