#!/usr/bin/perl

##
#
# This script makes new copies of all TCL files in a directory and rewrites
# variables in the process. Afterwards it deletes the original TCL files. The
# goal is to efficiently generate TCL for many simulations with different
# parameters.
#
# Example:	sim_generate_scripts.pl . nodes 10,12,30,40,50
#
# Copyright (c) 2007, 2008, 2009, 2010 Georg Wittenburg, Freie Universitaet
# Berlin (wittenbu@inf.fu-berlin.de). All rights reserved.
# Please refer to the COPYING file for detailed licensing information.
#
# $Id: sim_generate_scripts.pl 3313 2010-02-18 16:52:16Z gwitten $
#
##

use strict;

my $DELIMITER = "%%";

if(scalar(@ARGV) != 3) {
	print "Usage:\tsim_generate_scripts.pl tcl_directory variable_name values\n\n";
	exit(1);
}

my $tcl_directory = $ARGV[0];
my $variable_name = $ARGV[1];
my @values = split(/,/, $ARGV[2]);

if($tcl_directory !~ /\/\Z/o) {
	$tcl_directory .= '/';
}

print "Simulation files are in $tcl_directory.\n";
print "Variable name is \"$variable_name\", values are [".join(',', @values)."].\n";
print "\n";

print "Scanning for simulation scripts...\n";
my @tcl_scripts;
opendir(my $DIR, $tcl_directory) or die $!;
while(my $file = readdir($DIR)) {
	if($file =~ /\.tcl\Z/o) {
		push(@tcl_scripts, $tcl_directory.$file);
	}
}
closedir($DIR);
@tcl_scripts = sort(@tcl_scripts);
print "Found ".scalar(@tcl_scripts)." simulation script(s).\n";
print "\n";

my $pattern = $DELIMITER.$variable_name.$DELIMITER;
for my $tcl_script (@tcl_scripts) {
	for my $value (@values) {
		$tcl_script =~ /(.*)\.tcl\Z/o;
		my $new_script_name = $1."_".$variable_name."_".$value.".tcl";
		print "Creating $new_script_name...\n";
		open(my $IN_FILE, "<$tcl_script") or die $!;
		open(my $OUT_FILE, ">$new_script_name") or die $!;
		while(my $line = <$IN_FILE>) {
			$line =~ s/$pattern/$value/;
			print $OUT_FILE $line;
		}
		close($IN_FILE);
		close($OUT_FILE);
	}
	print "Deleting $tcl_script...\n";
	unlink $tcl_script;
}
print "Done.\n";
